@@ -3,12 +3,14 @@ package mustgather
 import (
 	"bufio"
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"math/rand"
 	"os"
 	"path"
 	"regexp"
+	"sort"
 	"strconv"
 	"strings"
 	"sync"
@@ -21,7 +23,7 @@ import (
 	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/apimachinery/pkg/util/errors"
+	kutilerrors "k8s.io/apimachinery/pkg/util/errors"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
 	"k8s.io/cli-runtime/pkg/genericiooptions"
@@ -38,6 +40,7 @@ import (
 	"k8s.io/kubectl/pkg/util/templates"
 	admissionapi "k8s.io/pod-security-admission/api"
 	"k8s.io/utils/exec"
+	utilptr "k8s.io/utils/ptr"
 
 	configclient "github.com/openshift/client-go/config/clientset/versioned"
 	imagev1client "github.com/openshift/client-go/image/clientset/versioned/typed/image/v1"
@@ -50,7 +53,13 @@ import (
 )
 
 const (
-	gatherContainerName = "gather"
+	gatherContainerName       = "gather"
+	unreachableTaintKey       = "node.kubernetes.io/unreachable"
+	notReadyTaintKey          = "node.kubernetes.io/not-ready"
+	controlPlaneNodeRoleLabel = "node-role.kubernetes.io/control-plane"
+	defaultMustGatherCommand  = "/usr/bin/gather"
+	defaultVolumePercentage   = 70
+	defaultSourceDir          = "/must-gather/"
 )
 
 var (
@@ -82,22 +91,52 @@ var (
 	`)
 
 	volumeUsageCheckerScript = `
-echo "volume percentage checker started....."
+echo "[disk usage checker] Started"
+target_dir="%s"
+usage_percentage_limit="%d"
 while true; do
-disk_usage=$(du -s "%s" | awk '{print $1}')
-disk_space=$(df -P "%s" | awk 'NR==2 {print $2}')
-usage_percentage=$(( (disk_usage * 100) / disk_space ))
-echo "volume usage percentage $usage_percentage"
-if [ "$usage_percentage" -gt "%d" ]; then
-	echo "Disk usage exceeds the volume percentage of %d for mounted directory. Exiting..."
-	# kill gathering process in gather container to prevent disk to use more.
-	pkill --signal SIGKILL -f %s
+usage_percentage=$(df -P "$target_dir" | awk 'NR==2 {print $5}' | sed 's/%%//')
+echo "[disk usage checker] Volume usage percentage: current = ${usage_percentage} ; allowed = ${usage_percentage_limit}"
+if [ "$usage_percentage" -gt "$usage_percentage_limit" ]; then
+	echo "[disk usage checker] Disk usage exceeds the volume percentage of ${usage_percentage_limit} for mounted directory, terminating..."
+	if [ "$HAVE_SESSION_TOOLS" = "true" ]; then
+		ps -o sess --no-headers | sort -u | while read sid; do
+			[[ "$sid" -eq "${$}" ]] && continue
+			pkill --signal SIGKILL --session "$sid"
+		done
+	else
+		kill 0
+	fi
 	exit 1
 fi
 sleep 5
 done`
 )
 
+// buildNodeAffinity builds a node affinity from the provided node hostnames.
+func buildNodeAffinity(nodeHostnames []string) *corev1.Affinity {
+	if len(nodeHostnames) == 0 {
+		return nil
+	}
+	return &corev1.Affinity{
+		NodeAffinity: &corev1.NodeAffinity{
+			RequiredDuringSchedulingIgnoredDuringExecution: &corev1.NodeSelector{
+				NodeSelectorTerms: []corev1.NodeSelectorTerm{
+					{
+						MatchExpressions: []corev1.NodeSelectorRequirement{
+							{
+								Key:      "kubernetes.io/hostname",
+								Operator: corev1.NodeSelectorOpIn,
+								Values:   nodeHostnames,
+							},
+						},
+					},
+				},
+			},
+		},
+	}
+}
+
 const (
 	// number of concurrent must-gather Pods to run if --all-images or multiple --image are provided
 	concurrentMG = 4
@@ -127,12 +166,12 @@ func NewMustGatherCommand(f kcmdutil.Factory, streams genericiooptions.IOStreams
 	cmd.Flags().BoolVar(&o.AllImages, "all-images", o.AllImages, `Collect must-gather using the default image for all Operators on the cluster annotated with `+mgAnnotation)
 	cmd.Flags().StringVar(&o.DestDir, "dest-dir", o.DestDir, "Set a specific directory on the local machine to write gathered data to.")
 	cmd.Flags().StringVar(&o.SourceDir, "source-dir", o.SourceDir, "Set the specific directory on the pod copy the gathered data from.")
-	cmd.Flags().StringVar(&o.timeoutStr, "timeout", "10m", "The length of time to gather data, like 5s, 2m, or 3h, higher than zero. Defaults to 10 minutes.")
+	cmd.Flags().StringVar(&o.timeoutStr, "timeout", "10m", "The length of time to wait for data gathering to complete, like 5s, 2m, or 3h, higher than zero. Defaults to 10 minutes. NOTE: This timeout only applies to the data gathering phase. After gathering completes, copying to the local destination will continue until finished.")
 	cmd.Flags().StringVar(&o.RunNamespace, "run-namespace", o.RunNamespace, "An existing privileged namespace where must-gather pods should run. If not specified a temporary namespace will be generated.")
-	cmd.Flags().Uint8Var(&o.VolumePercentage, "volume-percentage", o.VolumePercentage, "Specify maximum percentage of must-gather pod's allocated volume that can be used. If this limit is exceeded, must-gather will stop gathering, but still copy gathered data. Defaults to 30%.")
+	cmd.Flags().Uint8Var(&o.VolumePercentage, "volume-percentage", o.VolumePercentage, "Specify maximum percentage of must-gather pod's allocated volume that can be used. If this limit is exceeded, must-gather will stop gathering, but still copy gathered data.")
 	cmd.Flags().BoolVar(&o.Keep, "keep", o.Keep, "Do not delete temporary resources when command completes.")
 	cmd.Flags().MarkHidden("keep")
-	cmd.Flags().StringVar(&o.SinceTime, "since-time", o.SinceTime, "Only return logs after a specific date (RFC3339). Defaults to all logs. Plugins are encouraged but not required to support this. Only one of since-time / since may be used.")
+	cmd.Flags().StringVar(&o.SinceTime, "since-time", o.SinceTime, "Only return logs after a specific date (RFC3339). Defaults to all logs. Plugins are encouraged but not required to support this. Only one of since-time / since may be used. This may not be applied to all commands in must-gather image (e.g. not every command complies with RFC3339, the use might be limited, etc.).")
 	cmd.Flags().DurationVar(&o.Since, "since", o.Since, "Only return logs newer than a relative duration like 5s, 2m, or 3h. Defaults to all logs. Plugins are encouraged but not required to support this. Only one of since-time / since may be used.")
 
 	return cmd
@@ -140,10 +179,10 @@ func NewMustGatherCommand(f kcmdutil.Factory, streams genericiooptions.IOStreams
 
 func NewMustGatherOptions(streams genericiooptions.IOStreams) *MustGatherOptions {
 	opts := &MustGatherOptions{
-		SourceDir:        "/must-gather/",
+		SourceDir:        defaultSourceDir,
 		IOStreams:        streams,
 		Timeout:          10 * time.Minute,
-		VolumePercentage: 30,
+		VolumePercentage: defaultVolumePercentage,
 	}
 	opts.LogOut = opts.newPrefixWriter(streams.Out, "[must-gather      ] OUT", false, true)
 	opts.RawOut = opts.newPrefixWriter(streams.Out, "", false, false)
@@ -210,7 +249,7 @@ func (o *MustGatherOptions) Complete(f kcmdutil.Factory, cmd *cobra.Command, arg
 			return fmt.Errorf("--run-namespace %s", errStr)
 		}
 	}
-	if err := o.completeImages(); err != nil {
+	if err := o.completeImages(context.TODO()); err != nil {
 		return err
 	}
 	o.PrinterCreated, err = printers.NewTypeSetter(scheme.Scheme).WrapToPrinter(&printers.NamePrinter{Operation: "created"}, nil)
@@ -225,9 +264,9 @@ func (o *MustGatherOptions) Complete(f kcmdutil.Factory, cmd *cobra.Command, arg
 	return nil
 }
 
-func (o *MustGatherOptions) completeImages() error {
+func (o *MustGatherOptions) completeImages(ctx context.Context) error {
 	for _, imageStream := range o.ImageStreams {
-		if image, err := o.resolveImageStreamTagString(imageStream); err == nil {
+		if image, err := o.resolveImageStreamTagString(ctx, imageStream); err == nil {
 			o.Images = append(o.Images, image)
 		} else {
 			return fmt.Errorf("unable to resolve image stream '%v': %v", imageStream, err)
@@ -236,7 +275,7 @@ func (o *MustGatherOptions) completeImages() error {
 	if len(o.Images) == 0 || o.AllImages {
 		var image string
 		var err error
-		if image, err = o.resolveImageStreamTag("openshift", "must-gather", "latest"); err != nil {
+		if image, err = o.resolveImageStreamTag(ctx, "openshift", "must-gather", "latest"); err != nil {
 			o.log("%v\n", err)
 			image = "registry.redhat.io/openshift4/ose-must-gather:latest"
 		}
@@ -247,12 +286,12 @@ func (o *MustGatherOptions) completeImages() error {
 		pluginImages := make(map[string]struct{})
 		var err error
 
-		pluginImages, err = o.annotatedCSVs()
+		pluginImages, err = o.annotatedCSVs(ctx)
 		if err != nil {
 			return err
 		}
 
-		cos, err := o.ConfigClient.ConfigV1().ClusterOperators().List(context.TODO(), metav1.ListOptions{})
+		cos, err := o.ConfigClient.ConfigV1().ClusterOperators().List(ctx, metav1.ListOptions{})
 		if err != nil {
 			return err
 		}
@@ -273,7 +312,7 @@ func (o *MustGatherOptions) completeImages() error {
 	return nil
 }
 
-func (o *MustGatherOptions) annotatedCSVs() (map[string]struct{}, error) {
+func (o *MustGatherOptions) annotatedCSVs(ctx context.Context) (map[string]struct{}, error) {
 	csvGVR := schema.GroupVersionResource{
 		Group:    "operators.coreos.com",
 		Version:  "v1alpha1",
@@ -281,7 +320,7 @@ func (o *MustGatherOptions) annotatedCSVs() (map[string]struct{}, error) {
 	}
 	pluginImages := make(map[string]struct{})
 
-	csvs, err := o.DynamicClient.Resource(csvGVR).List(context.TODO(), metav1.ListOptions{})
+	csvs, err := o.DynamicClient.Resource(csvGVR).List(ctx, metav1.ListOptions{})
 	if err != nil {
 		return nil, err
 	}
@@ -295,12 +334,12 @@ func (o *MustGatherOptions) annotatedCSVs() (map[string]struct{}, error) {
 	return pluginImages, nil
 }
 
-func (o *MustGatherOptions) resolveImageStreamTagString(s string) (string, error) {
+func (o *MustGatherOptions) resolveImageStreamTagString(ctx context.Context, s string) (string, error) {
 	namespace, name, tag := parseImageStreamTagString(s)
 	if len(namespace) == 0 {
 		return "", fmt.Errorf("expected namespace/name:tag")
 	}
-	return o.resolveImageStreamTag(namespace, name, tag)
+	return o.resolveImageStreamTag(ctx, namespace, name, tag)
 }
 
 func parseImageStreamTagString(s string) (string, string, string) {
@@ -317,8 +356,8 @@ func parseImageStreamTagString(s string) (string, string, string) {
 	return namespace, name, tag
 }
 
-func (o *MustGatherOptions) resolveImageStreamTag(namespace, name, tag string) (string, error) {
-	imageStream, err := o.ImageClient.ImageStreams(namespace).Get(context.TODO(), name, metav1.GetOptions{})
+func (o *MustGatherOptions) resolveImageStreamTag(ctx context.Context, namespace, name, tag string) (string, error) {
+	imageStream, err := o.ImageClient.ImageStreams(namespace).Get(ctx, name, metav1.GetOptions{})
 	if err != nil {
 		return "", err
 	}
@@ -386,8 +425,8 @@ func (o *MustGatherOptions) Validate() error {
 	if o.VolumePercentage <= 0 || o.VolumePercentage > 100 {
 		return fmt.Errorf("invalid volume usage percentage, please specify a value between 0 and 100")
 	}
-	if o.VolumePercentage >= 80 {
-		klog.Warningf("volume percentage greater than or equal to 80 might cause filling up the disk space and have an impact on other components running on master")
+	if o.VolumePercentage >= 90 {
+		klog.Warningf("volume percentage greater than or equal to 90 might cause filling up the disk space and have an impact on other components running on master")
 	}
 
 	if len(o.SinceTime) > 0 && o.Since != 0 {
@@ -399,14 +438,136 @@ func (o *MustGatherOptions) Validate() error {
 			return fmt.Errorf("--since-time only accepts times matching RFC3339, eg '2006-01-02T15:04:05Z'")
 		}
 	}
-
 	return nil
 }
 
+func getNodeLastHeartbeatTime(node corev1.Node) *metav1.Time {
+	for _, cond := range node.Status.Conditions {
+		if cond.Type == corev1.NodeReady {
+			if !cond.LastHeartbeatTime.IsZero() {
+				return utilptr.To[metav1.Time](cond.LastHeartbeatTime)
+			}
+			return nil
+		}
+	}
+	return nil
+}
+
+func isNodeReadyByCondition(node corev1.Node) bool {
+	for _, cond := range node.Status.Conditions {
+		if cond.Type == corev1.NodeReady && cond.Status == corev1.ConditionTrue {
+			return true
+		}
+	}
+	return false
+}
+
+func isNodeReadyAndReachableByTaint(node corev1.Node) bool {
+	for _, taint := range node.Spec.Taints {
+		if taint.Key == unreachableTaintKey || taint.Key == notReadyTaintKey {
+			return false
+		}
+	}
+	return true
+}
+
+// getCandidateNodeNames identifies suitable nodes for constructing a list of
+// node names for a node affinity expression.
+func getCandidateNodeNames(nodes *corev1.NodeList, hasMaster bool) []string {
+	// Identify ready and reacheable nodes
+	var controlPlaneNodes, allControlPlaneNodes, workerNodes, unschedulableNodes, remainingNodes, selectedNodes []corev1.Node
+	for _, node := range nodes.Items {
+		if _, ok := node.Labels[controlPlaneNodeRoleLabel]; ok {
+			allControlPlaneNodes = append(allControlPlaneNodes, node)
+		}
+		if !isNodeReadyByCondition(node) || !isNodeReadyAndReachableByTaint(node) {
+			remainingNodes = append(remainingNodes, node)
+			continue
+		}
+		if node.Spec.Unschedulable {
+			unschedulableNodes = append(unschedulableNodes, node)
+			continue
+		}
+		if _, ok := node.Labels[controlPlaneNodeRoleLabel]; ok {
+			controlPlaneNodes = append(controlPlaneNodes, node)
+		} else {
+			workerNodes = append(workerNodes, node)
+		}
+	}
+
+	// INFO(ingvagabund): a single target node should be enough. Yet,
+	// it might help to provide more to allow the scheduler choose a more
+	// suitable node based on the cluster scheduling capabilities.
+
+	// hasMaster cause the must-gather pod to set a node selector targeting all
+	// control plane node. So there's no point of processing other than control
+	// plane nodes
+	if hasMaster {
+		if len(controlPlaneNodes) > 0 {
+			selectedNodes = controlPlaneNodes
+		} else {
+			selectedNodes = allControlPlaneNodes
+		}
+	} else {
+		// For hypershift case
+
+		// Order of preference:
+		// - ready and reachable control plane nodes first
+		// - then ready and reachable worker nodes
+		// - then ready and reachable unschedulable nodes
+		// - then any other node
+		selectedNodes = controlPlaneNodes // this will be very likely empty for hypershift
+		if len(selectedNodes) == 0 {
+			selectedNodes = workerNodes
+		}
+		if len(selectedNodes) == 0 {
+			// unschedulable nodes might still be better candidates than the remaining
+			// not ready and not reacheable nodes
+			selectedNodes = unschedulableNodes
+		}
+		if len(selectedNodes) == 0 {
+			// whatever is left for the last resort
+			selectedNodes = remainingNodes
+		}
+	}
+
+	// Sort nodes based on the cond.LastHeartbeatTime.Time to prefer nodes that
+	// reported their status most recently
+	sort.SliceStable(selectedNodes, func(i, j int) bool {
+		iTime := getNodeLastHeartbeatTime(selectedNodes[i])
+		jTime := getNodeLastHeartbeatTime(selectedNodes[j])
+		// iTime has no effect since all is sorted right
+		if jTime == nil {
+			return true
+		}
+		if iTime == nil {
+			return false
+		}
+		return jTime.Before(iTime)
+	})
+
+	// Limit the number of nodes to 10 at most to provide a sane list of nodes
+	nodeNames := []string{}
+	var nodeNamesSize = 0
+	for _, n := range selectedNodes {
+		nodeNames = append(nodeNames, n.Name)
+		nodeNamesSize++
+		if nodeNamesSize >= 10 {
+			break
+		}
+	}
+
+	return nodeNames
+}
+
 // Run creates and runs a must-gather pod
 func (o *MustGatherOptions) Run() error {
 	var errs []error
 
+	// The following context is being used for now until proper signal handling is implemented.
+	ctx, cancel := context.WithCancel(context.TODO())
+	defer cancel()
+
 	if err := os.MkdirAll(o.DestDir, os.ModePerm); err != nil {
 		// ensure the errors bubble up to BackupGathering method for display
 		errs = []error{err}
@@ -429,39 +590,52 @@ func (o *MustGatherOptions) Run() error {
 	}
 
 	// print at both the beginning and at the end.  This information is important enough to be in both spots.
-	o.PrintBasicClusterState(context.TODO())
+	o.PrintBasicClusterState(ctx)
 	defer func() {
-		fmt.Fprintf(o.RawOut, "\n\n")
-		fmt.Fprintf(o.RawOut, "Reprinting Cluster State:\n")
-		o.PrintBasicClusterState(context.TODO())
-	}()
-
-	// Due to 'stack unwiding', this should happen after 'clusterState' printing, to ensure that we always
-	//  print our ClusterState information.
-	runBackCollection := true
-	defer func() {
-		if !runBackCollection {
+		// Shortcircuit this in case the context is already cancelled.
+		if ctx.Err() != nil {
+			klog.Warning("Reprinting cluster state skipped, terminating...")
 			return
 		}
-		o.BackupGathering(context.TODO(), errs)
+		fmt.Fprintf(o.RawOut, "\n\n")
+		fmt.Fprintf(o.RawOut, "Reprinting Cluster State:\n")
+		o.PrintBasicClusterState(ctx)
+	}()
+
+	// Ensure resource cleanup unless instructed otherwise ...
+	// There is no context passed into the cleanup function as it's unrelated to the main context.
+	var cleanupNamespace func()
+	if !o.Keep {
+		defer func() {
+			if cleanupNamespace != nil {
+				cleanupNamespace()
+			}
+		}()
+	}
+
+	// Due to 'stack unwinding', this should happen after 'clusterState' printing, to ensure that we always
+	// print our ClusterState information.
+	runBackCollection := true
+	defer func() {
+		// Shortcircuit this in case the context is already cancelled.
+		if ctx.Err() != nil || !runBackCollection {
+			return
+		}
+		o.BackupGathering(ctx, errs)
 	}()
 
 	// Get or create "working" namespace ...
-	ns, cleanupNamespace, err := o.getNamespace()
+	var ns *corev1.Namespace
+	ns, cleanupNamespace, err = o.getNamespace(ctx)
 	if err != nil {
 		// ensure the errors bubble up to BackupGathering method for display
 		errs = []error{err}
 		return err
 	}
 
-	// ... ensure resource cleanup unless instructed otherwise ...
-	if !o.Keep {
-		defer cleanupNamespace()
-	}
-
 	// Prefer to run in master if there's any but don't be explicit otherwise.
 	// This enables the command to run by default in hypershift where there's no masters.
-	nodes, err := o.Client.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{
+	nodes, err := o.Client.CoreV1().Nodes().List(ctx, metav1.ListOptions{
 		LabelSelector: o.NodeSelector,
 	})
 	if err != nil {
@@ -469,12 +643,14 @@ func (o *MustGatherOptions) Run() error {
 	}
 	var hasMaster bool
 	for _, node := range nodes.Items {
-		if _, ok := node.Labels["node-role.kubernetes.io/master"]; ok {
+		if _, ok := node.Labels[controlPlaneNodeRoleLabel]; ok {
 			hasMaster = true
 			break
 		}
 	}
 
+	affinity := buildNodeAffinity(getCandidateNodeNames(nodes, hasMaster))
+
 	// ... and create must-gather pod(s)
 	var pods []*corev1.Pod
 	for _, image := range o.Images {
@@ -483,11 +659,11 @@ func (o *MustGatherOptions) Run() error {
 			line := fmt.Sprintf("unable to parse image reference %s: %v", image, err)
 			o.log(line)
 			// ensure the errors bubble up to BackupGathering method for display
-			errs = []error{fmt.Errorf(line)}
+			errs = []error{errors.New(line)}
 			return err
 		}
 		if o.NodeSelector != "" {
-			nodes, err := o.Client.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{
+			nodes, err := o.Client.CoreV1().Nodes().List(ctx, metav1.ListOptions{
 				LabelSelector: o.NodeSelector,
 			})
 			if err != nil {
@@ -496,17 +672,17 @@ func (o *MustGatherOptions) Run() error {
 				return err
 			}
 			for _, node := range nodes.Items {
-				pods = append(pods, o.newPod(node.Name, image, hasMaster))
+				pods = append(pods, o.newPod(node.Name, image, hasMaster, affinity))
 			}
 		} else {
 			if o.NodeName != "" {
-				if _, err := o.Client.CoreV1().Nodes().Get(context.TODO(), o.NodeName, metav1.GetOptions{}); err != nil {
+				if _, err := o.Client.CoreV1().Nodes().Get(ctx, o.NodeName, metav1.GetOptions{}); err != nil {
 					// ensure the errors bubble up to BackupGathering method for display
 					errs = []error{err}
 					return err
 				}
 			}
-			pods = append(pods, o.newPod(o.NodeName, image, hasMaster))
+			pods = append(pods, o.newPod(o.NodeName, image, hasMaster, affinity))
 		}
 	}
 
@@ -518,7 +694,7 @@ func (o *MustGatherOptions) Run() error {
 	}
 	defer o.logTimestamp()
 
-	queue := workqueue.NewRateLimitingQueue(workqueue.DefaultControllerRateLimiter())
+	queue := workqueue.NewTypedRateLimitingQueue[*corev1.Pod](workqueue.DefaultTypedControllerRateLimiter[*corev1.Pod]())
 
 	var wg sync.WaitGroup
 	errCh := make(chan error, len(pods))
@@ -528,6 +704,11 @@ func (o *MustGatherOptions) Run() error {
 	}
 	queue.ShutDownWithDrain()
 
+	go func() {
+		<-ctx.Done()
+		queue.ShutDown()
+	}()
+
 	wg.Add(concurrentMG)
 	for i := 0; i < concurrentMG; i++ {
 		go func() {
@@ -537,10 +718,14 @@ func (o *MustGatherOptions) Run() error {
 				if quit {
 					return
 				}
-				defer queue.Done(pod)
-				if err := o.processNextWorkItem(ns.Name, pod.(*corev1.Pod)); err != nil {
-					errCh <- err
-				}
+				func() {
+					// Make sure queue.Done is called inside the current iteration,
+					// not just when the whole worker thread exits.
+					defer queue.Done(pod)
+					if err := o.processNextWorkItem(ctx, ns.Name, pod); err != nil {
+						errCh <- err
+					}
+				}()
 			}
 		}()
 	}
@@ -564,13 +749,13 @@ func (o *MustGatherOptions) Run() error {
 		errs = append(errs, err)
 	}
 
-	return errors.NewAggregate(errs)
+	return kutilerrors.NewAggregate(errs)
 }
 
 // processNextWorkItem creates & processes the must-gather pod and returns error if any
-func (o *MustGatherOptions) processNextWorkItem(ns string, pod *corev1.Pod) error {
+func (o *MustGatherOptions) processNextWorkItem(ctx context.Context, ns string, pod *corev1.Pod) error {
 	var err error
-	pod, err = o.Client.CoreV1().Pods(ns).Create(context.TODO(), pod, metav1.CreateOptions{})
+	pod, err = o.Client.CoreV1().Pods(ns).Create(ctx, pod, metav1.CreateOptions{})
 	if err != nil {
 		return err
 	}
@@ -585,8 +770,7 @@ func (o *MustGatherOptions) processNextWorkItem(ns string, pod *corev1.Pod) erro
 			// it deletes this namespace and all the pods are removed by garbage collector.
 			// However, if user specifies namespace via `run-namespace`, these pods need to
 			// be deleted manually.
-			err = o.Client.CoreV1().Pods(o.RunNamespace).Delete(context.TODO(), pod.Name, metav1.DeleteOptions{})
-			if err != nil {
+			if err := o.Client.CoreV1().Pods(o.RunNamespace).Delete(ctx, pod.Name, metav1.DeleteOptions{}); err != nil {
 				klog.V(4).Infof("pod deletion error %v", err)
 			}
 		}()
@@ -595,19 +779,18 @@ func (o *MustGatherOptions) processNextWorkItem(ns string, pod *corev1.Pod) erro
 	log := o.newPodOutLogger(o.Out, pod.Name)
 
 	// wait for gather container to be running (gather is running)
-	if err := o.waitForGatherContainerRunning(pod); err != nil {
+	if err := o.waitForGatherContainerRunning(ctx, pod); err != nil {
 		log("gather did not start: %s", err)
-		return fmt.Errorf("gather did not start for pod %s: %s", pod.Name, err)
-
+		return fmt.Errorf("gather did not start for pod %s: %w", pod.Name, err)
 	}
 	// stream gather container logs
-	if err := o.getGatherContainerLogs(pod); err != nil {
+	if err := o.getGatherContainerLogs(ctx, pod); err != nil {
 		log("gather logs unavailable: %v", err)
 	}
 
 	// wait for pod to be running (gather has completed)
 	log("waiting for gather to complete")
-	if err := o.waitForGatherToComplete(pod); err != nil {
+	if err := o.waitForGatherToComplete(ctx, pod); err != nil {
 		log("gather never finished: %v", err)
 		if exiterr, ok := err.(*exec.CodeExitError); ok {
 			return exiterr
@@ -617,12 +800,12 @@ func (o *MustGatherOptions) processNextWorkItem(ns string, pod *corev1.Pod) erro
 
 	// copy the gathered files into the local destination dir
 	log("downloading gather output")
-	pod, err = o.Client.CoreV1().Pods(pod.Namespace).Get(context.TODO(), pod.Name, metav1.GetOptions{})
+	pod, err = o.Client.CoreV1().Pods(pod.Namespace).Get(ctx, pod.Name, metav1.GetOptions{})
 	if err != nil {
 		log("gather output not downloaded: %v\n", err)
 		return fmt.Errorf("unable to download output from pod %s: %s", pod.Name, err)
 	}
-	if err := o.copyFilesFromPod(pod); err != nil {
+	if err := o.copyFilesFromPod(ctx, pod); err != nil {
 		log("gather output not downloaded: %v\n", err)
 		return fmt.Errorf("unable to download output from pod %s: %s", pod.Name, err)
 	}
@@ -649,7 +832,7 @@ func (o *MustGatherOptions) logTimestamp() error {
 	return err
 }
 
-func (o *MustGatherOptions) copyFilesFromPod(pod *corev1.Pod) error {
+func (o *MustGatherOptions) copyFilesFromPod(ctx context.Context, pod *corev1.Pod) error {
 	streams := o.IOStreams
 	streams.Out = o.newPrefixWriter(streams.Out, fmt.Sprintf("[%s] OUT", pod.Name), false, true)
 	imageFolder := regexp.MustCompile("[^A-Za-z0-9]+").ReplaceAllString(pod.Status.ContainerStatuses[0].ImageID, "-")
@@ -677,7 +860,7 @@ func (o *MustGatherOptions) copyFilesFromPod(pod *corev1.Pod) error {
 			Container:  gatherContainerName,
 			Timestamps: true,
 		}
-		readCloser, err := o.Client.CoreV1().Pods(pod.Namespace).GetLogs(pod.Name, logOptions).Stream(context.TODO())
+		readCloser, err := o.Client.CoreV1().Pods(pod.Namespace).GetLogs(pod.Name, logOptions).Stream(ctx)
 		if err != nil {
 			return err
 		}
@@ -708,10 +891,10 @@ func (o *MustGatherOptions) copyFilesFromPod(pod *corev1.Pod) error {
 		err = rsyncOptions.RunRsync()
 		errs = append(errs, err)
 	}
-	return errors.NewAggregate(errs)
+	return kutilerrors.NewAggregate(errs)
 }
 
-func (o *MustGatherOptions) getGatherContainerLogs(pod *corev1.Pod) error {
+func (o *MustGatherOptions) getGatherContainerLogs(ctx context.Context, pod *corev1.Pod) error {
 	since2s := int64(2)
 	opts := &logs.LogsOptions{
 		Namespace:   pod.Namespace,
@@ -732,13 +915,15 @@ func (o *MustGatherOptions) getGatherContainerLogs(pod *corev1.Pod) error {
 		// gather script might take longer than the default API server time,
 		// so we should check if the gather script still runs and re-run logs
 		// thus we run this in a loop
+		//
+		// TODO: Use opts.RunLogsContext once Kubernetes v1.35 is available.
 		if err := opts.RunLogs(); err != nil {
 			return err
 		}
 
 		// to ensure we don't print all of history set since to past 2 seconds
 		opts.Options.(*corev1.PodLogOptions).SinceSeconds = &since2s
-		if done, _ := o.isGatherDone(pod); done {
+		if done, _ := o.isGatherDone(ctx, pod); done {
 			return nil
 		}
 		klog.V(4).Infof("lost logs, re-trying...")
@@ -769,15 +954,15 @@ func (o *MustGatherOptions) newPrefixWriter(out io.Writer, prefix string, ignore
 	return writer
 }
 
-func (o *MustGatherOptions) waitForGatherToComplete(pod *corev1.Pod) error {
-	return wait.PollImmediate(10*time.Second, o.Timeout, func() (bool, error) {
-		return o.isGatherDone(pod)
+func (o *MustGatherOptions) waitForGatherToComplete(ctx context.Context, pod *corev1.Pod) error {
+	return wait.PollUntilContextTimeout(ctx, 10*time.Second, o.Timeout, true, func(ctx context.Context) (bool, error) {
+		return o.isGatherDone(ctx, pod)
 	})
 }
 
-func (o *MustGatherOptions) isGatherDone(pod *corev1.Pod) (bool, error) {
+func (o *MustGatherOptions) isGatherDone(ctx context.Context, pod *corev1.Pod) (bool, error) {
 	var err error
-	if pod, err = o.Client.CoreV1().Pods(pod.Namespace).Get(context.TODO(), pod.Name, metav1.GetOptions{}); err != nil {
+	if pod, err = o.Client.CoreV1().Pods(pod.Namespace).Get(ctx, pod.Name, metav1.GetOptions{}); err != nil {
 		// at this stage pod should exist, we've been gathering container logs, so error if not found
 		if kerrors.IsNotFound(err) {
 			return true, err
@@ -809,10 +994,10 @@ func (o *MustGatherOptions) isGatherDone(pod *corev1.Pod) (bool, error) {
 	return false, nil
 }
 
-func (o *MustGatherOptions) waitForGatherContainerRunning(pod *corev1.Pod) error {
-	return wait.PollImmediate(10*time.Second, o.Timeout, func() (bool, error) {
+func (o *MustGatherOptions) waitForGatherContainerRunning(ctx context.Context, pod *corev1.Pod) error {
+	return wait.PollUntilContextTimeout(ctx, 10*time.Second, o.Timeout, true, func(ctx context.Context) (bool, error) {
 		var err error
-		if pod, err = o.Client.CoreV1().Pods(pod.Namespace).Get(context.TODO(), pod.Name, metav1.GetOptions{}); err == nil {
+		if pod, err = o.Client.CoreV1().Pods(pod.Namespace).Get(ctx, pod.Name, metav1.GetOptions{}); err == nil {
 			if len(pod.Status.ContainerStatuses) == 0 {
 				return false, nil
 			}
@@ -834,12 +1019,12 @@ func (o *MustGatherOptions) waitForGatherContainerRunning(pod *corev1.Pod) error
 	})
 }
 
-func (o *MustGatherOptions) getNamespace() (*corev1.Namespace, func(), error) {
+func (o *MustGatherOptions) getNamespace(ctx context.Context) (*corev1.Namespace, func(), error) {
 	if o.RunNamespace == "" {
-		return o.createTempNamespace()
+		return o.createTempNamespace(ctx)
 	}
 
-	ns, err := o.Client.CoreV1().Namespaces().Get(context.TODO(), o.RunNamespace, metav1.GetOptions{})
+	ns, err := o.Client.CoreV1().Namespaces().Get(ctx, o.RunNamespace, metav1.GetOptions{})
 	if err != nil {
 		return nil, nil, fmt.Errorf("retrieving namespace %q: %w", o.RunNamespace, err)
 	}
@@ -847,19 +1032,13 @@ func (o *MustGatherOptions) getNamespace() (*corev1.Namespace, func(), error) {
 	return ns, func() {}, nil
 }
 
-func (o *MustGatherOptions) createTempNamespace() (*corev1.Namespace, func(), error) {
-	ns, err := o.Client.CoreV1().Namespaces().Create(context.TODO(), newNamespace(), metav1.CreateOptions{})
+func (o *MustGatherOptions) createTempNamespace(ctx context.Context) (*corev1.Namespace, func(), error) {
+	ns, err := o.Client.CoreV1().Namespaces().Create(ctx, newNamespace(), metav1.CreateOptions{})
 	if err != nil {
 		return nil, nil, fmt.Errorf("creating temp namespace: %w", err)
 	}
 	o.PrinterCreated.PrintObj(ns, o.LogOut)
 
-	crb, err := o.Client.RbacV1().ClusterRoleBindings().Create(context.TODO(), newClusterRoleBinding(ns), metav1.CreateOptions{})
-	if err != nil {
-		return nil, nil, fmt.Errorf("creating temp clusterRoleBinding: %w", err)
-	}
-	o.PrinterCreated.PrintObj(crb, o.LogOut)
-
 	cleanup := func() {
 		if err := o.Client.CoreV1().Namespaces().Delete(context.TODO(), ns.Name, metav1.DeleteOptions{}); err != nil {
 			fmt.Printf("%v\n", err)
@@ -868,6 +1047,12 @@ func (o *MustGatherOptions) createTempNamespace() (*corev1.Namespace, func(), er
 		}
 	}
 
+	crb, err := o.Client.RbacV1().ClusterRoleBindings().Create(ctx, newClusterRoleBinding(ns), metav1.CreateOptions{})
+	if err != nil {
+		return nil, cleanup, fmt.Errorf("creating temp clusterRoleBinding: %w", err)
+	}
+	o.PrinterCreated.PrintObj(crb, o.LogOut)
+
 	return ns, cleanup, nil
 }
 
@@ -921,28 +1106,14 @@ func newClusterRoleBinding(ns *corev1.Namespace) *rbacv1.ClusterRoleBinding {
 	}
 }
 
-// newPod creates a pod with 2 containers with a shared volume mount:
-// - gather: init containers that run gather command
-// - copy: no-op container we can exec into
-func (o *MustGatherOptions) newPod(node, image string, hasMaster bool) *corev1.Pod {
+func defaultMustGatherPod(image string) *corev1.Pod {
 	zero := int64(0)
 
-	nodeSelector := map[string]string{
-		corev1.LabelOSStable: "linux",
-	}
-	if node == "" && hasMaster {
-		nodeSelector["node-role.kubernetes.io/master"] = ""
-	}
+	cleanedSourceDir := path.Clean(defaultSourceDir)
+	volumeUsageChecker := fmt.Sprintf(volumeUsageCheckerScript, cleanedSourceDir, defaultVolumePercentage)
+	podCmd := buildPodCommand(volumeUsageChecker, defaultMustGatherCommand)
 
-	executedCommand := "/usr/bin/gather"
-	if len(o.Command) > 0 {
-		executedCommand = strings.Join(o.Command, " ")
-	}
-
-	cleanedSourceDir := path.Clean(o.SourceDir)
-	volumeUsageChecker := fmt.Sprintf(volumeUsageCheckerScript, cleanedSourceDir, cleanedSourceDir, o.VolumePercentage, o.VolumePercentage, executedCommand)
-
-	ret := &corev1.Pod{
+	return &corev1.Pod{
 		ObjectMeta: metav1.ObjectMeta{
 			GenerateName: "must-gather-",
 			Labels: map[string]string{
@@ -950,7 +1121,6 @@ func (o *MustGatherOptions) newPod(node, image string, hasMaster bool) *corev1.P
 			},
 		},
 		Spec: corev1.PodSpec{
-			NodeName: node,
 			// This pod is ok to be OOMKilled but not preempted. Following the conventions mentioned at:
 			// https://github.com/openshift/enhancements/blob/master/CONVENTIONS.md#priority-classes
 			// so setting priority class to system-cluster-critical
@@ -969,8 +1139,7 @@ func (o *MustGatherOptions) newPod(node, image string, hasMaster bool) *corev1.P
 					Name:            gatherContainerName,
 					Image:           image,
 					ImagePullPolicy: corev1.PullIfNotPresent,
-					// always force disk flush to ensure that all data gathered is accessible in the copy container
-					Command: []string{"/bin/bash", "-c", fmt.Sprintf("%s & %s; sync", volumeUsageChecker, executedCommand)},
+					Command:         []string{"/bin/bash", "-c", podCmd},
 					Env: []corev1.EnvVar{
 						{
 							Name: "NODE_NAME",
@@ -1013,8 +1182,9 @@ func (o *MustGatherOptions) newPod(node, image string, hasMaster bool) *corev1.P
 					},
 				},
 			},
-			HostNetwork:                   o.HostNetwork,
-			NodeSelector:                  nodeSelector,
+			NodeSelector: map[string]string{
+				corev1.LabelOSStable: "linux",
+			},
 			TerminationGracePeriodSeconds: &zero,
 			Tolerations: []corev1.Toleration{
 				{
@@ -1026,6 +1196,49 @@ func (o *MustGatherOptions) newPod(node, image string, hasMaster bool) *corev1.P
 			},
 		},
 	}
+}
+
+// newPod creates a pod with 2 containers with a shared volume mount:
+// - gather: init containers that run gather command
+// - copy: no-op container we can exec into
+func (o *MustGatherOptions) newPod(node, image string, hasMaster bool, affinity *corev1.Affinity) *corev1.Pod {
+	executedCommand := defaultMustGatherCommand
+	if len(o.Command) > 0 {
+		executedCommand = strings.Join(o.Command, " ")
+	}
+
+	cleanedSourceDir := path.Clean(o.SourceDir)
+	volumeUsageChecker := fmt.Sprintf(volumeUsageCheckerScript, cleanedSourceDir, o.VolumePercentage)
+	podCmd := buildPodCommand(volumeUsageChecker, executedCommand)
+
+	ret := defaultMustGatherPod(image)
+	ret.Spec.Containers[0].Command = []string{"/bin/bash", "-c", podCmd}
+	ret.Spec.Containers[0].VolumeMounts = []corev1.VolumeMount{{
+		Name:      "must-gather-output",
+		MountPath: cleanedSourceDir,
+		ReadOnly:  false,
+	}}
+	ret.Spec.Containers[1].VolumeMounts = []corev1.VolumeMount{{
+		Name:      "must-gather-output",
+		MountPath: cleanedSourceDir,
+		ReadOnly:  false,
+	}}
+	ret.Spec.HostNetwork = o.HostNetwork
+	if node == "" && hasMaster {
+		ret.Spec.NodeSelector[controlPlaneNodeRoleLabel] = ""
+	}
+
+	if node != "" {
+		ret.Spec.NodeName = node
+	} else {
+		// Set affinity towards the most suitable nodes. E.g. to exclude
+		// nodes that if unreachable could cause the must-gather pod to stay
+		// in Pending state indefinitely. Please check getCandidateNodeNames
+		// function for more details about how the selection of the most
+		// suitable nodes is performed.
+		ret.Spec.Affinity = affinity
+	}
+
 	if o.HostNetwork {
 		// If a user specified hostNetwork he might have intended to perform
 		// packet captures on the host, for that we need to set the correct
@@ -1061,27 +1274,81 @@ func (o *MustGatherOptions) newPod(node, image string, hasMaster bool) *corev1.P
 // BackupGathering is called if the full must-gather has an error.  This is useful for making sure we get *something*
 // no matter what has failed.  It should be focused on universal openshift failures.
 func (o *MustGatherOptions) BackupGathering(ctx context.Context, errs []error) {
+	typeTargets := strings.Join([]string{
+		"clusterversion.v1.config.openshift.io",
+		"clusteroperators.v1.config.openshift.io",
+	}, ",")
+	namedTargets := []string{
+		"namespace/openshift-cluster-version",
+	}
+
 	fmt.Fprintf(o.ErrOut, "\n\n") // Space out the output
-	fmt.Fprintf(o.ErrOut, "Error running must-gather collection:\n    %v\n\n", errors.NewAggregate(errs))
-	fmt.Fprintf(o.ErrOut, "Falling back to `oc adm inspect clusteroperators.v1.config.openshift.io` to collect basic cluster information.\n")
+	fmt.Fprintf(o.ErrOut, "Error running must-gather collection:\n    %v\n\n", kutilerrors.NewAggregate(errs))
+	fmt.Fprintf(o.ErrOut, "Falling back to `oc adm inspect %s` to collect basic cluster types.\n", typeTargets)
 
 	streams := o.IOStreams
 	streams.Out = o.newPrefixWriter(streams.Out, fmt.Sprintf("[must-gather      ] OUT"), false, true)
+	destDir := path.Join(o.DestDir, fmt.Sprintf("inspect.local.%06d", rand.Int63()))
 
+	if err := runInspect(ctx, streams, rest.CopyConfig(o.Config), destDir, []string{typeTargets}); err != nil {
+		fmt.Fprintf(o.ErrOut, "error completing cluster type inspection: %v\n", err)
+	}
+
+	fmt.Fprintf(o.ErrOut, "Falling back to `oc adm inspect %s` to collect basic cluster named resources.\n", strings.Join(namedTargets, " "))
+
+	if err := runInspect(ctx, streams, rest.CopyConfig(o.Config), destDir, namedTargets); err != nil {
+		fmt.Fprintf(o.ErrOut, "error completing cluster named resource inspection: %v\n", err)
+	}
+	return
+}
+
+func runInspect(ctx context.Context, streams genericiooptions.IOStreams, config *rest.Config, destDir string, arguments []string) error {
 	inspectOptions := inspect.NewInspectOptions(streams)
-	inspectOptions.RESTConfig = rest.CopyConfig(o.Config)
-	inspectOptions.DestDir = path.Join(o.DestDir, fmt.Sprintf("inspect.local.%06d", rand.Int63()))
+	inspectOptions.RESTConfig = config
+	inspectOptions.DestDir = destDir
 
-	if err := inspectOptions.Complete([]string{"clusteroperators.v1.config.openshift.io"}); err != nil {
-		fmt.Fprintf(o.ErrOut, "error completing backup collection: %v\n", err)
-		return
+	if err := inspectOptions.Complete(arguments); err != nil {
+		return fmt.Errorf("error completing backup collection: %w", err)
 	}
 	if err := inspectOptions.Validate(); err != nil {
-		fmt.Fprintf(o.ErrOut, "error validating backup collection: %v\n", err)
-		return
+		return fmt.Errorf("error validating backup collection: %w", err)
 	}
-	if err := inspectOptions.Run(); err != nil {
-		fmt.Fprintf(o.ErrOut, "error running backup collection: %v\n", err)
-		return
+	if err := inspectOptions.RunContext(ctx); err != nil {
+		return fmt.Errorf("error running backup collection: %w", err)
 	}
+	return nil
+}
+
+func buildPodCommand(
+	volumeCheckerScript string,
+	gatherCommand string,
+) string {
+	var cmd strings.Builder
+
+	// Check if session management tools are available once and store in a variable.
+	cmd.WriteString("if command -v setsid >/dev/null 2>&1 && command -v ps >/dev/null 2>&1 && command -v pkill >/dev/null 2>&1; then\n")
+	cmd.WriteString("  HAVE_SESSION_TOOLS=true\n")
+	cmd.WriteString("else\n")
+	cmd.WriteString("  HAVE_SESSION_TOOLS=false\n")
+	cmd.WriteString("fi\n\n")
+
+	// Start the checker in the background.
+	cmd.WriteString(volumeCheckerScript)
+	cmd.WriteString(` & `)
+
+	// Start the gather command in a separate session if setsid, ps, and pkill are available.
+	// Fall back to simpler approach if any of these tools are not present (minimal images).
+	cmd.WriteString(`if [ "$HAVE_SESSION_TOOLS" = "true" ]; then`)
+	cmd.WriteString("\n  setsid -w bash <<-MUSTGATHER_EOF\n")
+	cmd.WriteString(gatherCommand)
+	cmd.WriteString("\nMUSTGATHER_EOF\n")
+	cmd.WriteString("else\n")
+	cmd.WriteString("  ")
+	cmd.WriteString(gatherCommand)
+	cmd.WriteString("\nfi; ")
+
+	// Make sure all changes are written to disk.
+	cmd.WriteString(`sync && echo 'Caches written to disk'`)
+
+	return cmd.String()
 }
