@@ -19,6 +19,8 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"net/url"
+	"os"
+	"slices"
 	"strings"
 	"testing"
 	"testing/fstest"
@@ -33,11 +35,13 @@ import (
 	restclient "k8s.io/client-go/rest"
 	clientgotesting "k8s.io/client-go/testing"
 	"k8s.io/client-go/tools/remotecommand"
+	utilexec "k8s.io/utils/exec"
 
 	"github.com/distribution/distribution/v3/manifest/schema2"
 	configv1 "github.com/openshift/api/config/v1"
 	imageapi "github.com/openshift/api/image/v1"
 	configv1fake "github.com/openshift/client-go/config/clientset/versioned/fake"
+	fakeoperatorconfig "github.com/openshift/client-go/operator/clientset/versioned/fake"
 	"github.com/openshift/library-go/pkg/image/dockerv1client"
 	"github.com/openshift/oc/pkg/cli/rsync"
 )
@@ -48,7 +52,6 @@ var (
   interfaces:
   - name: eth0
     macAddress: 00:b9:9b:c8:ac:f4`
-	defaultOutputName = "node.iso"
 )
 
 func TestValidate(t *testing.T) {
@@ -61,7 +64,6 @@ func TestValidate(t *testing.T) {
 		{
 			name:        "default",
 			nodesConfig: &defaultNodesConfigYaml,
-			outputName:  &defaultOutputName,
 		},
 		{
 			name:          "missing configuration file",
@@ -70,15 +72,8 @@ func TestValidate(t *testing.T) {
 		{
 			name:          "invalid configuration file",
 			nodesConfig:   strPtr("invalid: yaml\n\tfile"),
-			outputName:    &defaultOutputName,
 			expectedError: "config file nodes-config.yaml is not valid",
 		},
-		{
-			name:          "invalid output name",
-			nodesConfig:   &defaultNodesConfigYaml,
-			outputName:    strPtr(""),
-			expectedError: "--output-name cannot be empty",
-		},
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
@@ -91,9 +86,6 @@ func TestValidate(t *testing.T) {
 			o := &CreateOptions{
 				FSys: fakeFileSystem,
 			}
-			if tc.outputName != nil {
-				o.OutputName = *tc.outputName
-			}
 
 			err := o.Validate()
 
@@ -117,54 +109,80 @@ func strPtr(s string) *string {
 	return &s
 }
 
+func createCmdOutput(t *testing.T, r report) string {
+	t.Helper()
+	out, err := json.Marshal(r)
+	if err != nil {
+		t.Fatal(err)
+	}
+	return string(out)
+}
+
 func TestRun(t *testing.T) {
 	ClusterVersion_4_16_ObjectFn := func(repo string, manifestDigest string) []runtime.Object {
 		cvobj := defaultClusterVersionObjectFn(repo, manifestDigest)
 		clusterVersion := cvobj[0].(*configv1.ClusterVersion)
 		clusterVersion.Status.Desired.Version = "4.16.6-x86_64"
-
+		return cvobj
+	}
+	ClusterVersion_4_17_ObjectFn := func(repo string, manifestDigest string) []runtime.Object {
+		cvobj := defaultClusterVersionObjectFn(repo, manifestDigest)
+		clusterVersion := cvobj[0].(*configv1.ClusterVersion)
+		clusterVersion.Status.Desired.Version = "4.17.4-x86_64"
 		return cvobj
 	}
 
 	testCases := []struct {
-		name        string
-		nodesConfig string
-		assetsDir   string
-		outputName  string
+		name             string
+		nodesConfig      string
+		assetsDir        string
+		generatePXEFiles bool
 
 		objects          func(string, string) []runtime.Object
 		remoteExecOutput string
 
-		expectedOutputImage string
-		expectedError       string
-		expectedPod         func(t *testing.T, pod *corev1.Pod)
+		expectedErrorCode    int
+		expectedError        string
+		expectedPod          func(t *testing.T, pod *corev1.Pod)
+		expectedRsyncInclude []string
 	}{
 		{
-			name:        "default",
+			name:                 "default",
+			nodesConfig:          defaultNodesConfigYaml,
+			objects:              defaultClusterVersionObjectFn,
+			assetsDir:            "/my-working-dir",
+			generatePXEFiles:     false,
+			expectedRsyncInclude: []string{"*.iso"},
+		},
+		{
+			name:                 "default pxe",
+			nodesConfig:          defaultNodesConfigYaml,
+			objects:              defaultClusterVersionObjectFn,
+			assetsDir:            "/my-working-dir",
+			generatePXEFiles:     true,
+			expectedRsyncInclude: []string{"*.img", "*.*vmlinuz", "*.ipxe"},
+		},
+		{
+			name:        "node-joiner tool failure",
 			nodesConfig: defaultNodesConfigYaml,
 			objects:     defaultClusterVersionObjectFn,
+			remoteExecOutput: createCmdOutput(t, report{
+				stageHeader: stageHeader{
+					EndTime: time.Date(2024, 11, 14, 0, 0, 0, 0, time.UTC),
+				},
+				Result: &reportResult{
+					ExitCode:     127,
+					ErrorMessage: "Some error message",
+				},
+			}),
+			expectedErrorCode: 1,
+			expectedError:     `exit`,
 		},
 		{
-			name:                "command with options",
-			nodesConfig:         defaultNodesConfigYaml,
-			objects:             defaultClusterVersionObjectFn,
-			assetsDir:           "/my-working-dir",
-			outputName:          "node.iso",
-			expectedOutputImage: "/my-working-dir/node.iso",
-		},
-		{
-			name:             "node-joiner tool failure",
-			nodesConfig:      defaultNodesConfigYaml,
-			objects:          defaultClusterVersionObjectFn,
-			remoteExecOutput: "1",
-			expectedError:    `image generation error (exit code: 1)`,
-		},
-		{
-			name:             "node-joiner unsupported prior to 4.17",
-			nodesConfig:      defaultNodesConfigYaml,
-			objects:          ClusterVersion_4_16_ObjectFn,
-			remoteExecOutput: "1",
-			expectedError:    fmt.Sprintf("the 'oc adm node-image' command is only available for OpenShift versions %s and later", nodeJoinerMinimumSupportedVersion),
+			name:          "node-joiner unsupported prior to 4.17",
+			nodesConfig:   defaultNodesConfigYaml,
+			objects:       ClusterVersion_4_16_ObjectFn,
+			expectedError: fmt.Sprintf("the 'oc adm node-image' command is only available for OpenShift versions %s and later", nodeJoinerMinimumSupportedVersion),
 		},
 		{
 			name:          "missing cluster connection",
@@ -212,6 +230,12 @@ func TestRun(t *testing.T) {
 				}
 			},
 		},
+		{
+			name:             "basic report for ocp < 4.18",
+			nodesConfig:      defaultNodesConfigYaml,
+			objects:          ClusterVersion_4_17_ObjectFn,
+			remoteExecOutput: "0",
+		},
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
@@ -231,21 +255,34 @@ func TestRun(t *testing.T) {
 				objs = tc.objects(fakeReg.URL()[len("https://"):], fakeReg.fakeManifestDigest)
 			}
 
+			fakeRemoteExec.execOut = createCmdOutput(t, report{
+				stageHeader: stageHeader{
+					Identifier: "report-test",
+					EndTime:    time.Date(2024, 11, 14, 0, 0, 0, 0, time.UTC),
+				},
+				Result: &reportResult{
+					ExitCode: 0,
+				},
+			})
 			if tc.remoteExecOutput != "" {
 				fakeRemoteExec.execOut = tc.remoteExecOutput
 			}
 			// Create another fake for the copy action
 			fakeCp := &fakeCopier{}
 
+			var logBuffer bytes.Buffer
+
 			// Prepare the command options with all the fakes
 			o := &CreateOptions{
 				BaseNodeImageCommand: BaseNodeImageCommand{
-					IOStreams:      genericiooptions.NewTestIOStreamsDiscard(),
-					command:        createCommand,
-					ConfigClient:   configv1fake.NewSimpleClientset(objs...),
-					Client:         fakeClient,
-					Config:         fakeRestConfig,
-					remoteExecutor: fakeRemoteExec,
+					IOStreams:               genericiooptions.NewTestIOStreamsDiscard(),
+					command:                 createCommand,
+					ConfigClient:            configv1fake.NewSimpleClientset(objs...),
+					OpenshiftOperatorClient: fakeoperatorconfig.NewSimpleClientset(),
+					Client:                  fakeClient,
+					Config:                  fakeRestConfig,
+					remoteExecutor:          fakeRemoteExec,
+					LogOut:                  &logBuffer,
 				},
 				FSys: fakeFileSystem,
 				copyStrategy: func(o *rsync.RsyncOptions) rsync.CopyStrategy {
@@ -253,15 +290,16 @@ func TestRun(t *testing.T) {
 					return fakeCp
 				},
 
-				AssetsDir:  tc.assetsDir,
-				OutputName: tc.outputName,
+				AssetsDir:        tc.assetsDir,
+				GeneratePXEFiles: tc.generatePXEFiles,
+				fileWriter:       mockFileWriter{},
 			}
 			// Since the fake registry creates a self-signed cert, let's configure
 			// the command options accordingly
 			o.SecurityOptions.Insecure = true
 
 			err := o.Run()
-			assertContainerImageAndErrors(t, err, fakeReg, fakeClient, tc.expectedError, nodeJoinerContainer)
+			assertContainerImageAndErrors(t, err, fakeReg, fakeClient, tc.expectedErrorCode, tc.expectedError, nodeJoinerContainer)
 
 			// Perform additional checks on the generated node-joiner pod
 			if tc.expectedPod != nil {
@@ -270,14 +308,29 @@ func TestRun(t *testing.T) {
 			}
 
 			if tc.expectedError == "" {
-				if fakeCp.options.Destination.Path != tc.expectedOutputImage {
-					t.Errorf("expected %v, actual %v", tc.expectedOutputImage, fakeCp.options.Destination.Path)
+				if fakeCp.options.Destination.Path != tc.assetsDir {
+					t.Errorf("expected %v, actual %v", fakeCp.options.Destination.Path, tc.assetsDir)
+				}
+				if fakeCp.options.RsyncNoPerms == false {
+					t.Errorf("RsyncNoPerms is disabled")
+				}
+			}
+
+			for _, ext := range tc.expectedRsyncInclude {
+				if !slices.Contains(fakeCp.options.RsyncInclude, ext) {
+					t.Errorf("expected RSyncOptions to include %s, but doesn't", ext)
 				}
 			}
 		})
 	}
 }
 
+type mockFileWriter struct{}
+
+func (mockFileWriter) WriteFile(name string, data []byte, perm os.FileMode) error {
+	return nil
+}
+
 // fakeRegistry creates a fake Docker registry configured to serve the minimum
 // amount of data required to allow a successfull execution of the command to
 // retrieve the release info, and to extract the baremetal-installer pullspec.
@@ -485,6 +538,10 @@ type fakeRemoteExecutor struct {
 	execOut string
 }
 
+func (f *fakeRemoteExecutor) ExecuteWithContext(ctx context.Context, url *url.URL, config *restclient.Config, stdin io.Reader, stdout, stderr io.Writer, tty bool, terminalSizeQueue remotecommand.TerminalSizeQueue) error {
+	return f.Execute(url, config, stdin, stdout, stderr, tty, terminalSizeQueue)
+}
+
 func (f *fakeRemoteExecutor) Execute(url *url.URL, config *restclient.Config, stdin io.Reader, stdout, stderr io.Writer, tty bool, terminalSizeQueue remotecommand.TerminalSizeQueue) error {
 	f.url = url
 	stdout.Write([]byte(f.execOut))
@@ -548,7 +605,7 @@ func getTestPod(fakeClient *fake.Clientset, podName string) *corev1.Pod {
 	return pod
 }
 
-func assertContainerImageAndErrors(t *testing.T, runErr error, fakeReg *fakeRegistry, fakeClient *fake.Clientset, expectedError, podName string) {
+func assertContainerImageAndErrors(t *testing.T, runErr error, fakeReg *fakeRegistry, fakeClient *fake.Clientset, expectedErrorCode int, expectedError, podName string) {
 	if expectedError == "" {
 		if runErr != nil {
 			t.Fatalf("unexpected error: %v", runErr)
@@ -563,8 +620,13 @@ func assertContainerImageAndErrors(t *testing.T, runErr error, fakeReg *fakeRegi
 		if runErr == nil {
 			t.Fatalf("expected error not received: %s", expectedError)
 		}
-		if !strings.Contains(runErr.Error(), expectedError) {
-			t.Fatalf("expected error: %s, actual: %v", expectedError, runErr.Error())
+		if codeExitErr, ok := runErr.(utilexec.CodeExitError); ok {
+			if codeExitErr.Code != expectedErrorCode {
+				t.Fatalf("expected error code: %d, actual: %d", expectedErrorCode, codeExitErr.Code)
+			}
+		}
+		if runErr.Error() != expectedError {
+			t.Fatalf("expected error: %s, actual: %s", expectedError, runErr.Error())
 		}
 	}
 }
@@ -602,6 +664,13 @@ func TestCreateConfigFileFromFlags(t *testing.T) {
 			},
 			expectedConfigFile: expectedConfigFile,
 		},
+		{
+			name: "root device hints incorrect",
+			singleNodeCreateOptions: &singleNodeCreateOptions{
+				RootDeviceHints: "/dev/sda",
+			},
+			expectedError: "incorrect rootDeviceHints format provided: /dev/sda. Expected format: <hint name>:<value>, for example: deviceName:/dev/sda",
+		},
 		{
 			name: "ssh-key-path missing or incorrect",
 			singleNodeCreateOptions: &singleNodeCreateOptions{
