@@ -3,6 +3,7 @@ package imageprune
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -1368,7 +1369,7 @@ func deleteFromRegistry(registryClient *http.Client, url string) error {
 	// non-2xx/3xx response doesn't cause an error, so we need to check for it
 	// manually and return it to caller
 	if resp.StatusCode < http.StatusOK || resp.StatusCode >= http.StatusBadRequest {
-		return fmt.Errorf(resp.Status)
+		return errors.New(resp.Status)
 	}
 
 	if resp.StatusCode != http.StatusNoContent && resp.StatusCode != http.StatusAccepted {
