@@ -32,7 +32,6 @@ import (
 	"github.com/openshift/oc/pkg/cli/image/imagesource"
 	imagemanifest "github.com/openshift/oc/pkg/cli/image/manifest"
 	"github.com/openshift/oc/pkg/cli/image/workqueue"
-	"github.com/pkg/errors"
 )
 
 var (
@@ -349,9 +348,8 @@ func (o *ExtractOptions) Run(ctx context.Context) error {
 		}
 	}
 
-	tarEntryCallbacks := []extract.TarEntryFunc{}
-
 	var manifestErrs []error
+	// o.ExtractManifests implies o.File == ""
 	if o.ExtractManifests {
 		expectedProviderSpecKind := credRequestCloudProviderSpecKindMapping[o.Cloud]
 
@@ -380,7 +378,7 @@ func (o *ExtractOptions) Run(ctx context.Context) error {
 			include = newIncluder(inclusionConfig)
 		}
 
-		tarEntryCallbacks = append(tarEntryCallbacks, func(hdr *tar.Header, _ extract.LayerInfo, r io.Reader) (bool, error) {
+		opts.TarEntryCallback = func(hdr *tar.Header, _ extract.LayerInfo, r io.Reader) (bool, error) {
 			if hdr.Name == "image-references" && !o.CredentialsRequests {
 				buf := &bytes.Buffer{}
 				if _, err := io.Copy(buf, r); err != nil {
@@ -430,7 +428,7 @@ func (o *ExtractOptions) Run(ctx context.Context) error {
 			klog.V(4).Infof("Found manifest %s", hdr.Name)
 			ms, err := manifest.ParseManifests(r)
 			if err != nil {
-				manifestErrs = append(manifestErrs, errors.Wrapf(err, "error parsing %s", hdr.Name))
+				manifestErrs = append(manifestErrs, fmt.Errorf("error parsing %s: %w", hdr.Name, err))
 				return true, nil
 			}
 
@@ -454,7 +452,7 @@ func (o *ExtractOptions) Run(ctx context.Context) error {
 					if expectedProviderSpecKind != "" {
 						kind, _, err := unstructured.NestedString(m.Obj.Object, "spec", "providerSpec", "kind")
 						if err != nil {
-							return false, errors.Wrap(err, "error extracting cred request kind")
+							return false, fmt.Errorf("error extracting cred request kind: %w", err)
 						}
 						if kind != expectedProviderSpecKind {
 							continue
@@ -472,58 +470,34 @@ func (o *ExtractOptions) Run(ctx context.Context) error {
 			if o.Directory != "" {
 				out, err = os.Create(filepath.Join(o.Directory, hdr.Name))
 				if err != nil {
-					return false, errors.Wrapf(err, "error creating manifest in %s", hdr.Name)
+					return false, fmt.Errorf("error creating manifest in %s: %w", hdr.Name, err)
 				}
 			}
 			if out != nil {
 				for _, m := range manifestsToWrite {
 					yamlBytes, err := yaml.JSONToYAML(m.Raw)
 					if err != nil {
-						return false, errors.Wrapf(err, "error serializing manifest in %s", hdr.Name)
+						return false, fmt.Errorf("error serializing manifest in %s: %w", hdr.Name, err)
 					}
 					fmt.Fprintf(out, "---\n")
 					if _, err := out.Write(yamlBytes); err != nil {
-						return false, errors.Wrapf(err, "error writing manifest in %s", hdr.Name)
+						return false, fmt.Errorf("error writing manifest in %s: %w", hdr.Name, err)
 					}
 				}
 			}
 			return true, nil
-		})
+		}
 	}
 
 	fileFound := false
 	if o.File != "" {
-		tarEntryCallbacks = append(tarEntryCallbacks, func(hdr *tar.Header, _ extract.LayerInfo, r io.Reader) (bool, error) {
+		opts.TarEntryCallback = func(hdr *tar.Header, _ extract.LayerInfo, r io.Reader) (bool, error) {
 			if hdr.Name != o.File {
 				return true, nil
 			}
 			fileFound = true
 			_, err := io.Copy(o.Out, r)
 			return false, err
-		})
-	}
-
-	if len(tarEntryCallbacks) > 0 {
-		tarEntryCallbacksDone := make([]bool, len(tarEntryCallbacks))
-		opts.TarEntryCallback = func(hdr *tar.Header, layer extract.LayerInfo, r io.Reader) (bool, error) {
-			for i, callback := range tarEntryCallbacks {
-				if tarEntryCallbacksDone[i] {
-					continue
-				}
-				if cont, err := callback(hdr, layer, r); err != nil {
-					return cont, err
-				} else if !cont {
-					tarEntryCallbacksDone[i] = true
-				}
-			}
-
-			for _, done := range tarEntryCallbacksDone {
-				if !done {
-					return true, nil // still some callbacks that want to keep working
-				}
-			}
-
-			return false, nil
 		}
 	}
 
