@@ -66,19 +66,17 @@ func (p *Payload) Rewrite(allowTags bool, fn func(component string) imagereferen
 			continue
 		}
 		path := filepath.Join(p.path, file.Name())
-		data, err := ioutil.ReadFile(path)
+		data, err := os.ReadFile(path)
 		if err != nil {
 			return err
 		}
-		out, err := mapper(data)
-		if err != nil {
-			return fmt.Errorf("unable to rewrite the contents of %s: %v", path, err)
-		}
+		out := mapper(data)
+
 		if bytes.Equal(data, out) {
 			continue
 		}
 		klog.V(6).Infof("Rewrote\n%s\n\nto\n\n%s\n", string(data), string(out))
-		if err := ioutil.WriteFile(path, out, file.Mode()); err != nil {
+		if err := os.WriteFile(path, out, file.Mode()); err != nil {
 			return err
 		}
 	}
@@ -99,7 +97,7 @@ func (p *Payload) References() (*imageapi.ImageStream, error) {
 }
 
 func parseImageStream(path string) (*imageapi.ImageStream, error) {
-	data, err := ioutil.ReadFile(path)
+	data, err := os.ReadFile(path)
 	if os.IsNotExist(err) {
 		return nil, err
 	}
@@ -120,6 +118,8 @@ func readReleaseImageReferences(data []byte) (*imageapi.ImageStream, error) {
 	return is, nil
 }
 
+type SafeManifestMapper func(data []byte) []byte
+
 type ManifestMapper func(data []byte) ([]byte, error)
 
 func NewTransformFromImageStreamFile(path string, input *imageapi.ImageStream, allowMissingImages bool, errOut io.Writer) (ManifestMapper, error) {
@@ -140,11 +140,7 @@ func NewTransformFromImageStreamFile(path string, input *imageapi.ImageStream, a
 
 	versionMapper := NewComponentVersionsMapper(input.Name, versions, tagsByName)
 	return func(data []byte) ([]byte, error) {
-		data, err := imageMapper(data)
-		if err != nil {
-			return nil, err
-		}
-		return versionMapper(data)
+		return versionMapper(imageMapper(data))
 	}, nil
 }
 
@@ -234,8 +230,8 @@ type ImageReference struct {
 	TargetPullSpec   string
 }
 
-func NopManifestMapper(data []byte) ([]byte, error) {
-	return data, nil
+func NopManifestMapper(data []byte) []byte {
+	return data
 }
 
 // patternImageFormat attempts to match a docker pull spec by prefix (%s) and capture the
@@ -243,7 +239,7 @@ func NopManifestMapper(data []byte) ([]byte, error) {
 // end of file.
 const patternImageFormat = `([\W]|^)(%s)(:[\w][\w.-]{0,127}|@[A-Za-z][A-Za-z0-9]*(?:[-_+.][A-Za-z][A-Za-z0-9]*)*[:][[:xdigit:]]{2,})?([\s"']|$)`
 
-func NewImageMapper(images map[string]ImageReference) (ManifestMapper, error) {
+func NewImageMapper(images map[string]ImageReference) (SafeManifestMapper, error) {
 	repositories := make([]string, 0, len(images))
 	bySource := make(map[string]string)
 	for name, ref := range images {
@@ -263,7 +259,7 @@ func NewImageMapper(images map[string]ImageReference) (ManifestMapper, error) {
 	pattern := fmt.Sprintf(patternImageFormat, strings.Join(repositories, "|"))
 	re := regexp.MustCompile(pattern)
 
-	return func(data []byte) ([]byte, error) {
+	return func(data []byte) []byte {
 		out := re.ReplaceAllFunc(data, func(in []byte) []byte {
 			parts := re.FindSubmatch(in)
 			repository := string(parts[2])
@@ -275,7 +271,7 @@ func NewImageMapper(images map[string]ImageReference) (ManifestMapper, error) {
 			ref := images[name]
 
 			suffix := parts[3]
-			klog.V(2).Infof("found repository %q with locator %q in the input, switching to %q (from pattern %s)", string(repository), string(suffix), ref.TargetPullSpec, pattern)
+			klog.V(2).Infof("found repository %q with locator %q in the input, switching to %q (from pattern %s)", repository, string(suffix), ref.TargetPullSpec, pattern)
 			switch {
 			case len(suffix) == 0:
 				// we found a repository, but no tag or digest (implied latest), or we got an exact match
@@ -288,14 +284,14 @@ func NewImageMapper(images map[string]ImageReference) (ManifestMapper, error) {
 				return []byte(string(parts[1]) + ref.TargetPullSpec + string(parts[4]))
 			}
 		})
-		return out, nil
+		return out
 	}, nil
 }
 
 // exactImageFormat attempts to match a string on word boundaries
 const exactImageFormat = `\b%s\b`
 
-func NewExactMapper(mappings map[string]string) (ManifestMapper, error) {
+func NewExactMapper(mappings map[string]string) (SafeManifestMapper, error) {
 	patterns := make(map[string]*regexp.Regexp)
 	for from, to := range mappings {
 		pattern := fmt.Sprintf(exactImageFormat, regexp.QuoteMeta(from))
@@ -306,11 +302,11 @@ func NewExactMapper(mappings map[string]string) (ManifestMapper, error) {
 		patterns[to] = re
 	}
 
-	return func(data []byte) ([]byte, error) {
+	return func(data []byte) []byte {
 		for to, pattern := range patterns {
 			data = pattern.ReplaceAll(data, []byte(to))
 		}
-		return data, nil
+		return data
 	}, nil
 }
 
@@ -335,9 +331,15 @@ func ComponentReferencesForImageStream(is *imageapi.ImageStream) (func(string) i
 	}, nil
 }
 
-const (
-	componentVersionFormat = `([\W]|^)0\.0\.1-snapshot([a-z0-9\-]*)`
-)
+// NewSimpleVersionsMapper substitutes strings of the form 0.0.1-snapshot with releaseName, and
+// errors out if the manifest contains any other version references.
+//
+// If the input release name is not a semver, a request for `0.0.1-snapshot` will be left unmodified.
+func NewSimpleVersionsMapper(releaseName string) ManifestMapper {
+	return NewComponentVersionsMapper(releaseName, nil, nil)
+}
+
+var componentVersionRe = regexp.MustCompile(`(\W|^)0\.0\.1-snapshot([a-z0-9\-]*)`)
 
 // NewComponentVersionsMapper substitutes strings of the form 0.0.1-snapshot with releaseName and strings
 // of the form 0.0.1-snapshot-[component] with the version value located in versions, or returns an error.
@@ -352,17 +354,12 @@ func NewComponentVersionsMapper(releaseName string, versions ComponentVersions,
 	} else {
 		releaseName = ""
 	}
-	re, err := regexp.Compile(componentVersionFormat)
-	if err != nil {
-		return func([]byte) ([]byte, error) {
-			return nil, fmt.Errorf("component versions mapper regex: %v", err)
-		}
-	}
+
 	return func(data []byte) ([]byte, error) {
 		var missing []string
 		var conflicts []string
-		data = re.ReplaceAllFunc(data, func(part []byte) []byte {
-			matches := re.FindSubmatch(part)
+		data = componentVersionRe.ReplaceAllFunc(data, func(part []byte) []byte {
+			matches := componentVersionRe.FindSubmatch(part)
 			if matches == nil {
 				return part
 			}
@@ -415,7 +412,7 @@ var (
 	// reAllowedVersionKey limits the allowed component name to a strict subset
 	reAllowedVersionKey = regexp.MustCompile(`^[a-z0-9]+[\-a-z0-9]*[a-z0-9]+$`)
 	// reAllowedDisplayNameKey limits the allowed component name to a strict subset
-	reAllowedDisplayNameKey = regexp.MustCompile(`^[a-zA-Z0-9\-\:\s\(\)]+$`)
+	reAllowedDisplayNameKey = regexp.MustCompile(`^[a-zA-Z0-9\-:\s()]+$`)
 )
 
 // ComponentVersion includes the version and optional display name.
@@ -434,7 +431,7 @@ func (v ComponentVersion) String() string {
 // labels removed, but prerelease segments are preserved.
 type ComponentVersions map[string]ComponentVersion
 
-// OrderedKeys returns the keys in this map in lexigraphic order.
+// OrderedKeys returns the keys in this map in lexicographic order.
 func (v ComponentVersions) OrderedKeys() []string {
 	keys := make([]string, 0, len(v))
 	for k := range v {
