@@ -217,7 +217,7 @@ type NewOptions struct {
 	cleanupFns []func()
 }
 
-func (o *NewOptions) Complete(f kcmdutil.Factory, cmd *cobra.Command, args []string) error {
+func (o *NewOptions) Complete(f kcmdutil.Factory, _ *cobra.Command, args []string) error {
 	overlap := make(map[string]string)
 	var mappings []Mapping
 	for _, filename := range o.MappingFilenames {
@@ -346,10 +346,7 @@ func (o *NewOptions) Run(ctx context.Context) error {
 		o.AlwaysInclude = append(o.AlwaysInclude, o.ToImageBaseTag)
 	}
 
-	exclude := sets.NewString()
-	for _, s := range o.Exclude {
-		exclude.Insert(s)
-	}
+	exclude := sets.New[string](o.Exclude...)
 
 	metadata := make(map[string]imageData)
 	var ordered []string
@@ -785,17 +782,14 @@ func (o *NewOptions) Run(ctx context.Context) error {
 		return err
 	}
 
-	sort.Strings(operators)
-	switch {
-	case operators == nil:
-	case len(operators) == 0:
+	if len(operators) == 0 {
 		fmt.Fprintf(o.ErrOut, "warning: No operator metadata was found, no operators will be part of the release.\n")
 	}
 
 	return nil
 }
 
-func resolveImageStreamTagsToReferenceMode(inputIS, is *imageapi.ImageStream, referenceMode string, exclude sets.String) error {
+func resolveImageStreamTagsToReferenceMode(inputIS, is *imageapi.ImageStream, referenceMode string, exclude sets.Set[string]) error {
 	switch referenceMode {
 	case "public", "", "source":
 		forceExternal := referenceMode == "public" || referenceMode == ""
@@ -1314,33 +1308,12 @@ type nopCloser struct {
 
 func (_ nopCloser) Close() error { return nil }
 
-// writeNestedTarHeader writes a series of nested tar headers, starting with parts[0] and joining each
-// successive part, but only if the path does not exist already.
-func writeNestedTarHeader(tw *tar.Writer, parts []string, existing map[string]struct{}, hdr tar.Header) error {
-	for i := range parts {
-		componentDir := path.Join(parts[:i+1]...)
-		if _, ok := existing[componentDir]; ok {
-			continue
-		}
-		existing[componentDir] = struct{}{}
-		hdr.Name = componentDir
-		if err := tw.WriteHeader(&hdr); err != nil {
-			return err
-		}
-	}
-	return nil
-}
+const imageReferencesImageStreamFilename = "image-references"
 
 func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata, ordered []string, metadata map[string]imageData, allowMissingImages bool, verifiers []PayloadVerifier, errOut io.Writer) ([]string, error) {
-	var operators []string
-	directories := make(map[string]struct{})
-	files := make(map[string]int)
-
-	parts := []string{"release-manifests"}
-
 	// find the newest content date in the input
 	var newest time.Time
-	if err := iterateExtractedManifests(ordered, metadata, func(contents []os.FileInfo, name string, image imageData) error {
+	if err := iterateExtractedManifests(ordered, metadata, func(_ string, contents []os.FileInfo, _ string) error {
 		for _, fi := range contents {
 			if fi.IsDir() {
 				continue
@@ -1359,8 +1332,15 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 	gw := gzip.NewWriter(w)
 	tw := tar.NewWriter(gw)
 
+	manifestDestinationDir := "release-manifests"
 	// ensure the directory exists in the tar bundle
-	if err := writeNestedTarHeader(tw, parts, directories, tar.Header{Mode: 0777, ModTime: newest, Typeflag: tar.TypeDir}); err != nil {
+	releaseManifestsDirHdr := tar.Header{
+		Name:     manifestDestinationDir,
+		Mode:     0777,
+		ModTime:  newest,
+		Typeflag: tar.TypeDir,
+	}
+	if err := tw.WriteHeader(&releaseManifestsDirHdr); err != nil {
 		return nil, err
 	}
 
@@ -1369,7 +1349,15 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 	if err != nil {
 		return nil, err
 	}
-	if err := tw.WriteHeader(&tar.Header{Mode: 0444, ModTime: newest, Typeflag: tar.TypeReg, Name: path.Join(append(append([]string{}, parts...), "image-references")...), Size: int64(len(data))}); err != nil {
+
+	imageReferencesHdr := tar.Header{
+		Name:     path.Join(manifestDestinationDir, imageReferencesImageStreamFilename),
+		Mode:     0444,
+		ModTime:  newest,
+		Typeflag: tar.TypeReg,
+		Size:     int64(len(data)),
+	}
+	if err := tw.WriteHeader(&imageReferencesHdr); err != nil {
 		return nil, err
 	}
 	if _, err := tw.Write(data); err != nil {
@@ -1382,7 +1370,14 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 		if err != nil {
 			return nil, err
 		}
-		if err := tw.WriteHeader(&tar.Header{Mode: 0444, ModTime: newest, Typeflag: tar.TypeReg, Name: path.Join(append(append([]string{}, parts...), "release-metadata")...), Size: int64(len(data))}); err != nil {
+		releaseMetadataHdr := tar.Header{
+			Name:     path.Join(manifestDestinationDir, "release-metadata"),
+			Mode:     0444,
+			ModTime:  newest,
+			Typeflag: tar.TypeReg,
+			Size:     int64(len(data)),
+		}
+		if err := tw.WriteHeader(&releaseMetadataHdr); err != nil {
 			return nil, err
 		}
 		if _, err := tw.Write(data); err != nil {
@@ -1390,41 +1385,50 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 		}
 	}
 
+	var operators []string
+	files := make(map[string]int)
 	// read each directory, processing the manifests in order and updating the contents into the tar output
-	if err := iterateExtractedManifests(ordered, metadata, func(contents []os.FileInfo, name string, image imageData) error {
-		transform := NopManifestMapper
+	if err := iterateExtractedManifests(ordered, metadata, func(directory string, contents []os.FileInfo, operator string) error {
+		transform := NewSimpleVersionsMapper(is.Name)
 
-		if fi := takeFileByName(&contents, "image-references"); fi != nil {
-			path := filepath.Join(image.Directory, fi.Name())
-			klog.V(2).Infof("Perform image replacement based on inclusion of %s", path)
-			transform, err = NewTransformFromImageStreamFile(path, is, allowMissingImages, errOut)
-			if err != nil {
-				return fmt.Errorf("operator %q contained an invalid image-references file: %s", name, err)
+		// If there is an image-references file in the directory, we will need to replace image references in the manifests
+		// See: https://github.com/openshift/enhancements/blob/068e863988b58f70b5184e4ef49c0ad1c2913dfb/dev-guide/cluster-version-operator/dev/operators.md?plain=1#L157-L189
+		for _, fi := range contents {
+			if fi.Name() == imageReferencesImageStreamFilename && !fi.IsDir() {
+				imageReferencesPath := filepath.Join(directory, imageReferencesImageStreamFilename)
+				klog.V(2).Infof("Perform image replacement based on inclusion of %s", imageReferencesImageStreamFilename)
+				var err error
+				if transform, err = NewTransformFromImageStreamFile(imageReferencesPath, is, allowMissingImages, errOut); err != nil {
+					return fmt.Errorf("operator %q contained an invalid image-references file: %s", operator, err)
+				}
+				break
 			}
 		}
 
 		for _, fi := range contents {
-			if fi.IsDir() {
+			if fi.IsDir() || fi.Name() == imageReferencesImageStreamFilename {
 				continue
 			}
-			filename := fi.Name()
+
+			srcFilename := fi.Name()
+			dstFilename := srcFilename
 
 			// components that don't declare that they need to be part of the global order
 			// get put in a scoped bucket at the end. Only a few components should need to
 			// be in the global order.
-			if !strings.HasPrefix(filename, "0000_") {
-				filename = fmt.Sprintf("0000_50_%s_%s", name, filename)
+			if !strings.HasPrefix(srcFilename, "0000_") {
+				dstFilename = fmt.Sprintf("0000_50_%s_%s", operator, srcFilename)
 			}
-			if count, ok := files[filename]; ok {
-				ext := path.Ext(path.Base(filename))
-				files[filename] = count + 1
-				filename = fmt.Sprintf("%s_%d%s", strings.TrimSuffix(filename, ext), count+1, ext)
-				files[filename] = 1
-			} else {
-				files[filename] = 1
+			if count, ok := files[dstFilename]; ok {
+				ext := path.Ext(path.Base(dstFilename))
+				files[dstFilename] = count + 1
+				dstFilename = fmt.Sprintf("%s_%d%s", strings.TrimSuffix(dstFilename, ext), count+1, ext)
 			}
-			src := filepath.Join(image.Directory, fi.Name())
-			dst := path.Join(append(append([]string{}, parts...), filename)...)
+
+			files[dstFilename] = 1
+
+			src := filepath.Join(directory, srcFilename)
+			dst := path.Join(manifestDestinationDir, dstFilename)
 			klog.V(4).Infof("Copying %s to %s", src, dst)
 
 			data, err := os.ReadFile(src)
@@ -1433,7 +1437,7 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 			}
 
 			for _, fn := range verifiers {
-				if err := fn(filepath.Join(filepath.Base(image.Directory), fi.Name()), data); err != nil {
+				if err := fn(filepath.Join(filepath.Base(directory), srcFilename), data); err != nil {
 					return err
 				}
 			}
@@ -1442,7 +1446,14 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 			if err != nil {
 				return err
 			}
-			if err := tw.WriteHeader(&tar.Header{Mode: 0444, ModTime: fi.ModTime(), Typeflag: tar.TypeReg, Name: dst, Size: int64(len(modified))}); err != nil {
+			dstHdr := tar.Header{
+				Name:     dst,
+				Mode:     0444,
+				ModTime:  fi.ModTime(),
+				Typeflag: tar.TypeReg,
+				Size:     int64(len(modified)),
+			}
+			if err := tw.WriteHeader(&dstHdr); err != nil {
 				return err
 			}
 			klog.V(6).Infof("Writing payload to %s\n%s", dst, string(modified))
@@ -1450,7 +1461,7 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 				return err
 			}
 		}
-		operators = append(operators, name)
+		operators = append(operators, operator)
 		return nil
 	}); err != nil {
 		return nil, err
@@ -1465,7 +1476,9 @@ func writePayload(w io.Writer, is *imageapi.ImageStream, cm *CincinnatiMetadata,
 	return operators, nil
 }
 
-func iterateExtractedManifests(ordered []string, metadata map[string]imageData, fn func(contents []os.FileInfo, name string, image imageData) error) error {
+type handleOperatorManifestsFn func(directory string, contents []os.FileInfo, operator string) error
+
+func iterateExtractedManifests(ordered []string, metadata map[string]imageData, fn handleOperatorManifestsFn) error {
 	for _, name := range ordered {
 		image, ok := metadata[name]
 		if !ok {
@@ -1481,7 +1494,7 @@ func iterateExtractedManifests(ordered []string, metadata map[string]imageData,
 			continue
 		}
 
-		if err := fn(contents, name, image); err != nil {
+		if err := fn(image.Directory, contents, name); err != nil {
 			return err
 		}
 	}
@@ -1497,26 +1510,6 @@ func hasTag(tags []imageapi.TagReference, tag string) *imageapi.TagReference {
 	return nil
 }
 
-func pruneEmptyDirectories(dir string) error {
-	return filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
-		if err != nil {
-			return err
-		}
-		if !info.IsDir() {
-			return nil
-		}
-		names, err := os.ReadDir(path)
-		if err != nil {
-			return err
-		}
-		if len(names) > 0 {
-			return nil
-		}
-		klog.V(4).Infof("Component %s does not have any manifests", path)
-		return os.Remove(path)
-	})
-}
-
 type Mapping struct {
 	Source      string
 	Destination string
@@ -1579,17 +1572,6 @@ func parseFile(filename string, overlap map[string]string) ([]Mapping, error) {
 	return fileMappings, nil
 }
 
-func takeFileByName(files *[]os.FileInfo, name string) os.FileInfo {
-	for i, fi := range *files {
-		if fi.IsDir() || fi.Name() != name {
-			continue
-		}
-		*files = append((*files)[:i], (*files)[i+1:]...)
-		return fi
-	}
-	return nil
-}
-
 type PayloadVerifier func(filename string, data []byte) error
 
 func pruneUnreferencedImageStreams(out io.Writer, is *imageapi.ImageStream, metadata map[string]imageData, include []string) error {
