@@ -5,6 +5,7 @@ import (
 	"io"
 	"math"
 	"strings"
+	"text/tabwriter"
 	"text/template"
 	"time"
 
@@ -37,10 +38,15 @@ type operators struct {
 	Degraded int
 	// Updated is the count of operators that updated its version, no matter its conditions
 	Updated int
-	// Updating is the count of operators that have not updated its version and are with Progressing=True
-	Updating int
 	// Waiting is the count of operators that have not updated its version and are with Progressing=False
 	Waiting int
+	// Updating is the collection of cluster operators that are currently being updated
+	Updating []UpdatingClusterOperator
+}
+
+type UpdatingClusterOperator struct {
+	Name      string
+	Condition *v1.ClusterOperatorStatusCondition
 }
 
 func (o operators) StatusSummary() string {
@@ -55,10 +61,11 @@ func (o operators) StatusSummary() string {
 }
 
 type versions struct {
-	target            string
-	previous          string
-	isTargetInstall   bool
-	isPreviousPartial bool
+	target               string
+	previous             string
+	isTargetInstall      bool
+	isPreviousPartial    bool
+	isMultiArchMigration bool
 }
 
 func (v versions) String() string {
@@ -68,17 +75,22 @@ func (v versions) String() string {
 	if v.isPreviousPartial {
 		return fmt.Sprintf("%s (from incomplete %s)", v.target, v.previous)
 	}
+	if v.isMultiArchMigration {
+		return fmt.Sprintf("%s to Multi-Architecture", v.target)
+	}
 	return fmt.Sprintf("%s (from %s)", v.target, v.previous)
 }
 
 type controlPlaneStatusDisplayData struct {
-	Assessment        assessmentState
-	Completion        float64
-	Duration          time.Duration
-	EstDuration       time.Duration
-	EstTimeToComplete time.Duration
-	Operators         operators
-	TargetVersion     versions
+	Assessment           assessmentState
+	Completion           float64
+	CompletionAt         time.Time
+	Duration             time.Duration
+	EstDuration          time.Duration
+	EstTimeToComplete    time.Duration
+	Operators            operators
+	TargetVersion        versions
+	IsMultiArchMigration bool
 }
 
 const (
@@ -136,7 +148,7 @@ func coInsights(name string, available *v1.ClusterOperatorStatusCondition, degra
 	return insights
 }
 
-func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOperator, at time.Time) (controlPlaneStatusDisplayData, []updateInsight) {
+func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOperator, mcoImagePullSpec string, at time.Time) (controlPlaneStatusDisplayData, []updateInsight) {
 	var displayData controlPlaneStatusDisplayData
 	var insights []updateInsight
 
@@ -158,13 +170,17 @@ func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOpera
 		}
 		insights = append(insights, insight)
 	} else if c.Status != v1.ConditionFalse {
+		verb := "is"
+		if c.Status == v1.ConditionUnknown {
+			verb = "may be"
+		}
 		insight := updateInsight{
 			startedAt: c.LastTransitionTime.Time,
 			scope:     updateInsightScope{scopeType: scopeTypeControlPlane, resources: []scopeResource{{kind: cvGroupKind, name: cv.Name}}},
 			impact: updateInsightImpact{
 				level:       warningImpactLevel,
 				impactType:  updateStalledImpactType,
-				summary:     fmt.Sprintf("Cluster Version %s is failing to proceed with the update (%s)", cv.Name, c.Reason),
+				summary:     fmt.Sprintf("Cluster Version %s %s failing to proceed with the update (%s)", cv.Name, verb, c.Reason),
 				description: c.Message,
 			},
 			remediation: updateInsightRemediation{reference: "https://github.com/openshift/runbooks/blob/master/alerts/cluster-monitoring-operator/ClusterOperatorDegraded.md"},
@@ -189,10 +205,24 @@ func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOpera
 		}
 
 		var updated bool
-		for _, version := range operator.Status.Versions {
-			if version.Name == "operator" && version.Version == targetVersion {
-				updated = true
-				break
+		var mcoOperatorImageUpgrading bool
+		if operator.Name == "machine-config" {
+			for _, version := range operator.Status.Versions {
+				if version.Name == "operator-image" {
+					if mcoImagePullSpec != version.Version {
+						mcoOperatorImageUpgrading = true
+						break
+					}
+				}
+			}
+		}
+
+		if operator.Name != "machine-config" || !mcoOperatorImageUpgrading {
+			for _, version := range operator.Status.Versions {
+				if version.Name == "operator" && version.Version == targetVersion {
+					updated = true
+					break
+				}
 			}
 		}
 
@@ -222,7 +252,8 @@ func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOpera
 				lastObservedProgress = progressing.LastTransitionTime.Time
 			}
 			if !updated && progressing.Status == v1.ConditionTrue {
-				displayData.Operators.Updating++
+				displayData.Operators.Updating = append(displayData.Operators.Updating,
+					UpdatingClusterOperator{Name: operator.Name, Condition: progressing})
 			}
 			if !updated && progressing.Status == v1.ConditionFalse {
 				displayData.Operators.Waiting++
@@ -267,10 +298,11 @@ func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOpera
 		}
 		toLastObservedProgress = lastObservedProgress.Sub(started)
 		if currentHistoryItem.State == v1.CompletedUpdate {
-			updatingFor = currentHistoryItem.CompletionTime.Time.Sub(started)
+			displayData.CompletionAt = currentHistoryItem.CompletionTime.Time
 		} else {
-			updatingFor = at.Sub(started)
+			displayData.CompletionAt = at
 		}
+		updatingFor = displayData.CompletionAt.Sub(started)
 		// precision to seconds when under 60s
 		if updatingFor > 10*time.Minute {
 			displayData.Duration = updatingFor.Round(time.Minute)
@@ -281,6 +313,7 @@ func assessControlPlaneStatus(cv *v1.ClusterVersion, operators []v1.ClusterOpera
 
 	versionData, versionInsights := versionsFromHistory(cv.Status.History, cvScope, controlPlaneCompleted)
 	displayData.TargetVersion = versionData
+	displayData.IsMultiArchMigration = versionData.isMultiArchMigration
 	insights = append(insights, versionInsights...)
 
 	coCompletion := float64(displayData.Operators.Updated) / float64(displayData.Operators.Total)
@@ -362,6 +395,12 @@ func timewiseComplete(coCompletion float64) float64 {
 	}
 }
 
+func multiArchMigration(history []v1.UpdateHistory) bool {
+	return len(history) > 1 &&
+		history[0].Version == history[1].Version &&
+		history[0].Image != history[1].Image
+}
+
 func versionsFromHistory(history []v1.UpdateHistory, cvScope scopeResource, controlPlaneCompleted bool) (versions, []updateInsight) {
 	versionData := versions{
 		target:   "unknown",
@@ -377,6 +416,7 @@ func versionsFromHistory(history []v1.UpdateHistory, cvScope scopeResource, cont
 	if len(history) > 1 {
 		versionData.previous = history[1].Version
 		versionData.isPreviousPartial = history[1].State == v1.PartialUpdate
+		versionData.isMultiArchMigration = multiArchMigration(history)
 	}
 
 	var insights []updateInsight
@@ -413,7 +453,10 @@ func versionsFromHistory(history []v1.UpdateHistory, cvScope scopeResource, cont
 	return versionData, insights
 }
 
-func vagueUnder(actual, estimated time.Duration) string {
+func vagueUnder(actual, estimated time.Duration, isMultiArchMigration bool) string {
+	if isMultiArchMigration {
+		return "N/A for Multi-Architecture Migration"
+	}
 	threshold := 10 * time.Minute
 	switch {
 	case actual < -10*time.Minute:
@@ -425,19 +468,73 @@ func vagueUnder(actual, estimated time.Duration) string {
 	}
 }
 
+func commaJoin(elems []UpdatingClusterOperator) string {
+	var names []string
+	for _, e := range elems {
+		names = append(names, e.Name)
+	}
+	return strings.Join(names, ", ")
+}
+
 var controlPlaneStatusTemplate = template.Must(
 	template.New("controlPlaneStatus").
-		Funcs(template.FuncMap{"shortDuration": shortDuration, "vagueUnder": vagueUnder}).
+		Funcs(template.FuncMap{"shortDuration": shortDuration, "vagueUnder": vagueUnder, "commaJoin": commaJoin}).
 		Parse(controlPlaneStatusTemplateRaw))
 
-func (d *controlPlaneStatusDisplayData) Write(f io.Writer) error {
-	return controlPlaneStatusTemplate.Execute(f, d)
+func (d *controlPlaneStatusDisplayData) Write(f io.Writer, detailed bool, now time.Time) error {
+	if d.Operators.Updated == d.Operators.Total {
+		_, err := f.Write([]byte(fmt.Sprintf("= Control Plane =\nUpdate to %s successfully completed at %s (duration: %s)\n", d.TargetVersion.target, d.CompletionAt.UTC().Format(time.RFC3339), shortDuration(d.Duration))))
+		return err
+	}
+	if err := controlPlaneStatusTemplate.Execute(f, d); err != nil {
+		return err
+	}
+	if detailed && len(d.Operators.Updating) > 0 {
+		table := tabwriter.NewWriter(f, 0, 0, 3, ' ', 0)
+		f.Write([]byte("\nUpdating Cluster Operators"))
+		_, _ = table.Write([]byte("\nNAME\tSINCE\tREASON\tMESSAGE\n"))
+		for _, o := range d.Operators.Updating {
+			reason := o.Condition.Reason
+			if reason == "" {
+				reason = "-"
+			}
+			lines := strings.Split(o.Condition.Message, "\n")
+			_, _ = table.Write([]byte(o.Name + "\t"))
+			_, _ = table.Write([]byte(shortDuration(now.Sub(o.Condition.LastTransitionTime.Time)) + "\t"))
+			_, _ = table.Write([]byte(ellipsis(reason, 20) + "\t"))
+			for i, line := range lines {
+				if i == 0 {
+					_, _ = table.Write([]byte(line + "\n"))
+				} else {
+					_, _ = table.Write([]byte("\t\t\t" + line + "\n"))
+				}
+			}
+		}
+		if err := table.Flush(); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+func ellipsis(s string, maxLen int) string {
+	runes := []rune(s)
+	if len(runes) <= maxLen {
+		return s
+	}
+	if maxLen < 3 {
+		maxLen = 3
+	}
+	return string(runes[0:maxLen-3]) + "..."
 }
 
 const controlPlaneStatusTemplateRaw = `= Control Plane =
 Assessment:      {{ .Assessment }}
 Target Version:  {{ .TargetVersion }}
-Completion:      {{ printf "%.0f" .Completion }}% ({{ .Operators.Updated }} operators updated, {{ .Operators.Updating }} updating, {{ .Operators.Waiting }} waiting)
-Duration:        {{ shortDuration .Duration }}{{ if .EstTimeToComplete }} (Est. Time Remaining: {{ vagueUnder .EstTimeToComplete .EstDuration }}){{ end }}
-Operator Status: {{ .Operators.StatusSummary }}
+{{ with commaJoin .Operators.Updating -}}
+Updating:        {{ . }}
+{{ end -}}
+Completion:      {{ printf "%.0f" .Completion }}% ({{ .Operators.Updated }} operators updated, {{ len .Operators.Updating }} updating, {{ .Operators.Waiting }} waiting)
+Duration:        {{ shortDuration .Duration }}{{ if .EstTimeToComplete }} (Est. Time Remaining: {{ vagueUnder .EstTimeToComplete .EstDuration .IsMultiArchMigration }}){{ end }}
+Operator Health: {{ .Operators.StatusSummary }}
 `
