@@ -70,7 +70,7 @@ func (c *coBuilder) progressing(status configv1.ConditionStatus, optionFuncs ...
 			c.operator.Status.Conditions[i].Status = status
 			c.operator.Status.Conditions[i].Reason = "ProgressingTowardsDesired"
 			c.operator.Status.Conditions[i].Message = "Operand is operated by operator"
-			c.operator.Status.Conditions[i].LastTransitionTime = metav1.Now()
+			c.operator.Status.Conditions[i].LastTransitionTime = metav1.Date(2023, 12, 1, 23, 23, 0, 0, time.UTC)
 			for _, f := range optionFuncs {
 				f(&c.operator.Status.Conditions[i])
 			}
@@ -175,7 +175,14 @@ func TestAssessControlPlaneStatus_Operators(t *testing.T) {
 				co("one").operator,
 				co("two").progressing(configv1.ConditionTrue).operator,
 			},
-			expected: operators{Total: 2, Updating: 1, Waiting: 1},
+			expected: operators{Total: 2, Waiting: 1, Updating: []UpdatingClusterOperator{{Name: "two", Condition: &configv1.ClusterOperatorStatusCondition{
+				Type:               "Progressing",
+				Status:             "True",
+				LastTransitionTime: metav1.Date(2023, 12, 1, 23, 23, 0, 0, time.UTC),
+				Reason:             "ProgressingTowardsDesired",
+				Message:            "Operand is operated by operator",
+			},
+			}}},
 		},
 		{
 			name: "one out of two not available",
@@ -230,7 +237,14 @@ func TestAssessControlPlaneStatus_Operators(t *testing.T) {
 					available(configv1.ConditionFalse).
 					progressing(configv1.ConditionTrue).operator,
 			},
-			expected: operators{Total: 2, Unavailable: 1, Updating: 1, Waiting: 1},
+			expected: operators{Total: 2, Unavailable: 1, Waiting: 1, Updating: []UpdatingClusterOperator{{Name: "two", Condition: &configv1.ClusterOperatorStatusCondition{
+				Type:               "Progressing",
+				Status:             "True",
+				LastTransitionTime: metav1.Date(2023, 12, 1, 23, 23, 0, 0, time.UTC),
+				Reason:             "ProgressingTowardsDesired",
+				Message:            "Operand is operated by operator",
+			},
+			}}},
 		},
 		{
 			name: "one upgraded",
@@ -245,7 +259,7 @@ func TestAssessControlPlaneStatus_Operators(t *testing.T) {
 
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			actual, insights := assessControlPlaneStatus(&cvFixture, tc.operators, time.Now())
+			actual, insights := assessControlPlaneStatus(&cvFixture, tc.operators, "", time.Now())
 			if diff := cmp.Diff(tc.expected, actual.Operators, cmp.AllowUnexported(operators{})); diff != "" {
 				t.Errorf("%s, actual output differs from expected:\n%s", tc.name, diff)
 			}
@@ -258,6 +272,74 @@ func TestAssessControlPlaneStatus_Operators(t *testing.T) {
 	}
 }
 
+func TestAssessControlPlaneStatus_insights(t *testing.T) {
+	testCases := []struct {
+		name     string
+		cv       *configv1.ClusterVersion
+		expected []updateInsight
+	}{
+		{
+			name: "cv: something went wrong",
+			cv: &configv1.ClusterVersion{
+				ObjectMeta: metav1.ObjectMeta{Name: "version"},
+				Status: configv1.ClusterVersionStatus{
+					Desired: configv1.Release{Version: "new"},
+					Conditions: []configv1.ClusterOperatorStatusCondition{{
+						Type:   clusterStatusFailing,
+						Status: configv1.ConditionTrue,
+						Reason: "something went wrong",
+					}},
+				},
+			},
+			expected: []updateInsight{
+				{
+					scope: updateInsightScope{scopeType: "ControlPlane", resources: []scopeResource{{kind: scopeGroupKind{group: "config.openshift.io", kind: "ClusterVersion"}, name: "version"}}},
+					impact: updateInsightImpact{
+						level:      warningImpactLevel,
+						impactType: "Update Stalled",
+						summary:    "Cluster Version version is failing to proceed with the update (something went wrong)",
+					},
+					remediation: updateInsightRemediation{reference: "https://github.com/openshift/runbooks/blob/master/alerts/cluster-monitoring-operator/ClusterOperatorDegraded.md"},
+				},
+			},
+		},
+		{
+			name: "cv: SlowClusterOperator",
+			cv: &configv1.ClusterVersion{
+				ObjectMeta: metav1.ObjectMeta{Name: "version"},
+				Status: configv1.ClusterVersionStatus{
+					Desired: configv1.Release{Version: "new"},
+					Conditions: []configv1.ClusterOperatorStatusCondition{{
+						Type:   clusterStatusFailing,
+						Status: configv1.ConditionUnknown,
+						Reason: "SlowClusterOperator",
+					}},
+				},
+			},
+			expected: []updateInsight{
+				{
+					scope: updateInsightScope{scopeType: "ControlPlane", resources: []scopeResource{{kind: scopeGroupKind{group: "config.openshift.io", kind: "ClusterVersion"}, name: "version"}}},
+					impact: updateInsightImpact{
+						level:      warningImpactLevel,
+						impactType: "Update Stalled",
+						summary:    "Cluster Version version may be failing to proceed with the update (SlowClusterOperator)",
+					},
+					remediation: updateInsightRemediation{reference: "https://github.com/openshift/runbooks/blob/master/alerts/cluster-monitoring-operator/ClusterOperatorDegraded.md"},
+				},
+			},
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			_, insights := assessControlPlaneStatus(tc.cv, nil, "", time.Now())
+			if diff := cmp.Diff(tc.expected, insights, allowUnexportedInsightStructs); diff != "" {
+				t.Errorf("%s: expected insights differs from actual:\n%s", tc.name, diff)
+			}
+		})
+	}
+}
+
 func TestAssessControlPlaneStatus_Estimate(t *testing.T) {
 	now := time.Now()
 	minutesAgo := [250]time.Time{}
@@ -370,7 +452,7 @@ func TestAssessControlPlaneStatus_Estimate(t *testing.T) {
 				},
 			}
 			expectedEstCompletion := estimateCompletion(time.Hour, tc.assumedToLastProgress, now.Sub(tc.started), tc.assumedClusterOperatorCompleted)
-			actual, _ := assessControlPlaneStatus(cv, tc.operators, now)
+			actual, _ := assessControlPlaneStatus(cv, tc.operators, "", now)
 			if diff := cmp.Diff(expectedEstCompletion, actual.EstTimeToComplete); diff != "" {
 				t.Errorf("estimate to finish differs:\n%s", diff)
 			}
@@ -434,7 +516,7 @@ func TestAssessControlPlaneStatus_Completion(t *testing.T) {
 
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			actual, _ := assessControlPlaneStatus(&cvFixture, tc.operators, time.Now())
+			actual, _ := assessControlPlaneStatus(&cvFixture, tc.operators, "", time.Now())
 			if diff := cmp.Diff(tc.expectedCompletion, actual.Completion, cmpopts.EquateApprox(0, 0.1)); diff != "" {
 				t.Errorf("expected completion %f, got %f", tc.expectedCompletion, actual.Completion)
 			}
@@ -506,7 +588,7 @@ func TestAssessControlPlaneStatus_Duration(t *testing.T) {
 			cv := cvFixture.DeepCopy()
 			cv.Status.History = append(cv.Status.History, tc.firstHistoryItem)
 
-			actual, _ := assessControlPlaneStatus(cv, nil, now)
+			actual, _ := assessControlPlaneStatus(cv, nil, "", now)
 			if diff := cmp.Diff(tc.expectedDuration, actual.Duration); diff != "" {
 				t.Errorf("expected completion %s, got %s", tc.expectedDuration, actual.Duration)
 			}
