@@ -146,21 +146,29 @@ func ellipsizeNames(message string, name string) string {
 	return strings.Replace(message, name, "<node>", -1)
 }
 
-func assessNodesStatus(cv *configv1.ClusterVersion, pool mcfgv1.MachineConfigPool, nodes []corev1.Node, machineConfigs []mcfgv1.MachineConfig) ([]nodeDisplayData, []updateInsight) {
+func assessNodesStatus(cv *configv1.ClusterVersion, pool mcfgv1.MachineConfigPool, nodes []corev1.Node, machineConfigs []mcfgv1.MachineConfig, multiArchMigrationInProgress bool) ([]nodeDisplayData, []updateInsight) {
 	var nodesStatusData []nodeDisplayData
 	var insights []updateInsight
 	for _, node := range nodes {
+		desiredConfig, ok := node.Annotations[mco.DesiredMachineConfigAnnotationKey]
 		currentVersion, foundCurrent := getOpenShiftVersionOfMachineConfig(machineConfigs, node.Annotations[mco.CurrentMachineConfigAnnotationKey])
-		desiredVersion, foundDesired := getOpenShiftVersionOfMachineConfig(machineConfigs, node.Annotations[mco.DesiredMachineConfigAnnotationKey])
+		desiredVersion, foundDesired := getOpenShiftVersionOfMachineConfig(machineConfigs, desiredConfig)
+
+		lns := mco.NewLayeredNodeState(&node)
+		isUnavailable := lns.IsUnavailable(&pool)
 
-		isUnavailable := isNodeUnavailable(node, pool)
 		isDegraded := isNodeDegraded(node)
-		isUpdated := foundCurrent && isLatestUpdateHistoryVersionEqualTo(cv.Status.History, currentVersion)
+		annotationMatched := node.Annotations[mco.CurrentMachineConfigAnnotationKey] == desiredConfig
+		desiredAnnotationUpdated := desiredConfig == pool.Spec.Configuration.Name
+		isUpdated := foundCurrent && isLatestUpdateHistoryVersionEqualTo(cv.Status.History, currentVersion) &&
+			// The following condition is to handle the multi-arch migration because the version number stays the same there
+			(!multiArchMigrationInProgress || !ok || (annotationMatched && desiredAnnotationUpdated))
 
 		// foundCurrent makes sure we don't blip phase "updating" for nodes that we are not sure
 		// of their actual phase, even though the conservative assumption is that the node is
 		// at least updating or is updated.
-		isUpdating := !isUpdated && foundCurrent && foundDesired && isLatestUpdateHistoryVersionEqualTo(cv.Status.History, desiredVersion)
+		isUpdating := !isUpdated && foundCurrent && foundDesired && isLatestUpdateHistoryVersionEqualTo(cv.Status.History, desiredVersion) &&
+			(!annotationMatched && desiredAnnotationUpdated)
 
 		phase := calculatePhase(pool, node, isUpdating, isUpdated)
 		var estimate string
@@ -186,10 +194,12 @@ func assessNodesStatus(cv *configv1.ClusterVersion, pool mcfgv1.MachineConfigPoo
 			estimate = "-"
 		}
 
-		var message string
+		var message, insightSummary, insightDescription string
 		if isUnavailable && !isUpdating {
 			assessment = nodeAssessmentUnavailable
-			message = "Node is unavailable" // TODO: Consider bubbling up the exact reason for --details
+			message = lns.GetUnavailableReason()
+			insightSummary = lns.GetUnavailableMessage()
+			insightDescription = lns.GetUnavailableDescription()
 			estimate = "?"
 			if isUpdated {
 				estimate = "-"
@@ -199,13 +209,14 @@ func assessNodesStatus(cv *configv1.ClusterVersion, pool mcfgv1.MachineConfigPoo
 		if isDegraded {
 			assessment = nodeAssessmentDegraded
 			message = node.Annotations[mco.MachineConfigDaemonReasonAnnotationKey]
+			insightDescription = message
 			estimate = "?"
 			if isUpdated {
 				estimate = "-"
 			}
 		}
 
-		insights = append(insights, nodeInsights(pool, node, message, isUnavailable, isUpdating, isDegraded)...)
+		insights = append(insights, nodeInsights(pool, node.Name, insightSummary, insightDescription, lns.SeriouslyUnavailable(), isUpdating, isDegraded, lns.GetUnavailableSince())...)
 
 		nodesStatusData = append(nodesStatusData, nodeDisplayData{
 			Name:          node.Name,
@@ -275,11 +286,6 @@ func isLatestUpdateHistoryVersionEqualTo(history []configv1.UpdateHistory, versi
 	return false
 }
 
-func isNodeUnavailable(node corev1.Node, pool mcfgv1.MachineConfigPool) bool {
-	lns := mco.NewLayeredNodeState(&node)
-	return lns.IsUnavailable(&pool)
-}
-
 func isNodeDegraded(node corev1.Node) bool {
 	// Inspired by: https://github.com/openshift/machine-config-operator/blob/master/pkg/controller/node/status.go
 	if node.Annotations == nil {
@@ -334,7 +340,7 @@ func mcdUpdatingStateToPhase(state string) nodePhase {
 	}
 }
 
-func nodeInsights(pool mcfgv1.MachineConfigPool, node corev1.Node, reason string, isUnavailable, isUpdating, isDegraded bool) []updateInsight {
+func nodeInsights(pool mcfgv1.MachineConfigPool, node, summary, description string, isUnavailable, isUpdating, isDegraded bool, unavailableSince time.Time) []updateInsight {
 	var insights []updateInsight
 	scope := scopeTypeWorkerPool
 	if pool.Name == "master" {
@@ -343,16 +349,16 @@ func nodeInsights(pool mcfgv1.MachineConfigPool, node corev1.Node, reason string
 	nodeGroupKind := scopeGroupKind{kind: nodeKind}
 	if isUnavailable && !isUpdating {
 		insights = append(insights, updateInsight{
-			startedAt: time.Time{},
+			startedAt: unavailableSince,
 			scope: updateInsightScope{
 				scopeType: scope,
-				resources: []scopeResource{{kind: nodeGroupKind, name: node.Name}},
+				resources: []scopeResource{{kind: nodeGroupKind, name: node}},
 			},
 			impact: updateInsightImpact{
 				level:       warningImpactLevel,
 				impactType:  updateSpeedImpactType,
-				summary:     fmt.Sprintf("Node %s is unavailable", node.Name),
-				description: reason,
+				summary:     summary,
+				description: description,
 			},
 			remediation: updateInsightRemediation{
 				reference: "https://docs.openshift.com/container-platform/latest/post_installation_configuration/machine-configuration-tasks.html#understanding-the-machine-config-operator",
@@ -364,13 +370,13 @@ func nodeInsights(pool mcfgv1.MachineConfigPool, node corev1.Node, reason string
 			startedAt: time.Time{},
 			scope: updateInsightScope{
 				scopeType: scope,
-				resources: []scopeResource{{kind: nodeGroupKind, name: node.Name}},
+				resources: []scopeResource{{kind: nodeGroupKind, name: node}},
 			},
 			impact: updateInsightImpact{
 				level:       errorImpactLevel,
 				impactType:  updateStalledImpactType,
-				summary:     fmt.Sprintf("Node %s is degraded", node.Name),
-				description: reason,
+				summary:     fmt.Sprintf("Node %s is degraded", node),
+				description: description,
 			},
 			remediation: updateInsightRemediation{
 				reference: "https://docs.openshift.com/container-platform/latest/post_installation_configuration/machine-configuration-tasks.html#understanding-the-machine-config-operator",
@@ -483,10 +489,15 @@ func writePools(w io.Writer, workerPoolsStatusData []poolDisplayData) {
 			_, _ = tabw.Write([]byte(fmt.Sprintf("%d Total", pool.NodesOverview.Total) + "\n"))
 		} else {
 			_, _ = tabw.Write([]byte(pool.Assessment + "\t"))
-			_, _ = tabw.Write([]byte(fmt.Sprintf("%.0f%%", pool.Completion) + "\t"))
-			_, _ = tabw.Write([]byte(fmt.Sprintf("%d Total, %d Available, %d Progressing, %d Outdated, %d Draining, %d Excluded, %d Degraded",
-				pool.NodesOverview.Total, pool.NodesOverview.Available, pool.NodesOverview.Progressing, pool.NodesOverview.Outdated,
-				pool.NodesOverview.Draining, pool.NodesOverview.Excluded, pool.NodesOverview.Degraded) + "\n"))
+			_, _ = tabw.Write([]byte(fmt.Sprintf("%.0f%% (%d/%d)", pool.Completion, pool.NodesOverview.Total-pool.NodesOverview.Outdated, pool.NodesOverview.Total) + "\t"))
+			status := fmt.Sprintf("%d Available, %d Progressing, %d Draining", pool.NodesOverview.Available, pool.NodesOverview.Progressing, pool.NodesOverview.Draining)
+			for k, v := range map[string]int{"Excluded": pool.NodesOverview.Excluded, "Degraded": pool.NodesOverview.Degraded} {
+				if v > 0 {
+					status = fmt.Sprintf("%s, %d %s", status, v, k)
+				}
+			}
+			status = fmt.Sprintf("%s\n", status)
+			_, _ = tabw.Write([]byte(status))
 		}
 	}
 	tabw.Flush()
@@ -497,6 +508,10 @@ func (pool *poolDisplayData) WriteNodes(w io.Writer, detailed bool) {
 		return
 	}
 	if pool.Name == mco.MachineConfigPoolMaster {
+		if pool.Completion == 100 {
+			fmt.Fprintf(w, "\nAll control plane nodes successfully updated to %s\n", pool.Nodes[0].Version)
+			return
+		}
 		fmt.Fprintf(w, "\nControl Plane Nodes")
 	} else {
 		fmt.Fprintf(w, "\nWorker Pool Nodes: %s", pool.Name)
