@@ -247,21 +247,42 @@ func (n *nodeBuilder) degraded_draining(currentConfig, desiredConfig, reason str
 
 var nodeGroupKind = scopeGroupKind{group: corev1.GroupName, kind: "Node"}
 
-// TODO: Potentially create a more robust builder in the future
-var updateInsightControlPlaneNodeUnavailable = updateInsight{
-	impact: updateInsightImpact{
-		level:       warningImpactLevel,
-		impactType:  updateSpeedImpactType,
-		summary:     "Node a is unavailable",
-		description: "Node is unavailable",
-	},
-	scope: updateInsightScope{
-		scopeType: scopeTypeControlPlane,
-		resources: []scopeResource{{kind: nodeGroupKind, name: "a"}},
-	},
-	remediation: updateInsightRemediation{
-		reference: "https://docs.openshift.com/container-platform/latest/post_installation_configuration/machine-configuration-tasks.html#understanding-the-machine-config-operator",
-	},
+type updateInsightControlPlaneNodeUnavailableBuilder interface {
+	SetDescription(description string) updateInsightControlPlaneNodeUnavailableBuilder
+	Build() updateInsight
+}
+
+type fakeUpdateInsightControlPlaneNodeUnavailableBuilder struct {
+	updateInsight updateInsight
+}
+
+func newUpdateInsightControlPlaneNodeUnavailableBuilder() updateInsightControlPlaneNodeUnavailableBuilder {
+	return &fakeUpdateInsightControlPlaneNodeUnavailableBuilder{
+		updateInsight: updateInsight{
+			impact: updateInsightImpact{
+				level:       warningImpactLevel,
+				impactType:  updateSpeedImpactType,
+				summary:     "Node a is unavailable",
+				description: "Node is unavailable",
+			},
+			scope: updateInsightScope{
+				scopeType: scopeTypeControlPlane,
+				resources: []scopeResource{{kind: nodeGroupKind, name: "a"}},
+			},
+			remediation: updateInsightRemediation{
+				reference: "https://docs.openshift.com/container-platform/latest/post_installation_configuration/machine-configuration-tasks.html#understanding-the-machine-config-operator",
+			},
+		},
+	}
+}
+
+func (b *fakeUpdateInsightControlPlaneNodeUnavailableBuilder) SetDescription(description string) updateInsightControlPlaneNodeUnavailableBuilder {
+	b.updateInsight.impact.description = description
+	return b
+}
+
+func (b *fakeUpdateInsightControlPlaneNodeUnavailableBuilder) Build() updateInsight {
+	return b.updateInsight
 }
 
 func Test_assessNodesStatus(t *testing.T) {
@@ -532,14 +553,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStateUpdated,
 					Version:       newOCPVersion,
 					Estimate:      "-",
-					Message:       "Node is unavailable",
+					Message:       "Machine Config Daemon is processing the node",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     true,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "updated node is missing an MCD annotation machineconfiguration.openshift.io/currentConfig",
@@ -558,14 +578,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       "",
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Machine Config Daemon is processing the node",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     false,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "draining node is missing an MCD annotation machineconfiguration.openshift.io/desiredDrain",
@@ -632,14 +651,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       oldOCPVersion,
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Machine Config Daemon is processing the node",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     false,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "draining node is missing an MCD annotation machineconfiguration.openshift.io/state",
@@ -682,14 +700,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStateUpdated,
 					Version:       newOCPVersion,
 					Estimate:      "-",
-					Message:       "Node is unavailable",
+					Message:       "Machine Config Daemon is processing the node",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     true,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "updating node is missing MCD annotations - desiredConfig and state",
@@ -708,14 +725,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       oldOCPVersion,
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Node is marked unschedulable",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     false,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "updating node is missing MCD annotations - currentConfig and state",
@@ -734,14 +750,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       "",
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Node is marked unschedulable",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     false,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "updating node is missing MCD annotations - currentConfig and desiredConfig",
@@ -760,14 +775,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       "",
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Node is marked unschedulable",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     false,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "node without MCD annotations",
@@ -786,14 +800,13 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       "",
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Machine Config Daemon is processing the node",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     false,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{updateInsightControlPlaneNodeUnavailable},
 		},
 		{
 			name: "node is updated but unavailable",
@@ -812,34 +825,18 @@ func Test_assessNodesStatus(t *testing.T) {
 					Phase:         phaseStateUpdated,
 					Version:       newOCPVersion,
 					Estimate:      "-",
-					Message:       "Node is unavailable",
+					Message:       "Node is marked unschedulable",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
 					isUpdated:     true,
 				},
 			},
-			expectedUpdateInsight: []updateInsight{
-				{
-					impact: updateInsightImpact{
-						level:       warningImpactLevel,
-						impactType:  updateSpeedImpactType,
-						summary:     "Node a is unavailable",
-						description: "Node is unavailable",
-					},
-					scope: updateInsightScope{
-						scopeType: scopeTypeControlPlane,
-						resources: []scopeResource{{kind: nodeGroupKind, name: "a"}},
-					},
-					remediation: updateInsightRemediation{
-						reference: "https://docs.openshift.com/container-platform/latest/post_installation_configuration/machine-configuration-tasks.html#understanding-the-machine-config-operator",
-					},
-				},
-			}},
+		},
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			nodeDisplayData, updateInsight := assessNodesStatus(tc.args.cv, tc.args.pool, tc.args.nodes, tc.args.machineConfigs)
+			nodeDisplayData, updateInsight := assessNodesStatus(tc.args.cv, tc.args.pool, tc.args.nodes, tc.args.machineConfigs, false)
 			if diff := cmp.Diff(tc.expectedNodeDisplayData, nodeDisplayData, allowUnexportedWorkerPools); diff != "" {
 				t.Errorf("nodeDisplayData differ from expected:\n%s", diff)
 			}
@@ -919,7 +916,7 @@ func Test_assessNodesStatus_DisplayData_Sorting(t *testing.T) {
 					Phase:         phaseStatePending,
 					Version:       oldOCPVersion,
 					Estimate:      "?",
-					Message:       "Node is unavailable",
+					Message:       "Node is marked unschedulable",
 					isUnavailable: true,
 					isDegraded:    false,
 					isUpdating:    false,
@@ -954,7 +951,7 @@ func Test_assessNodesStatus_DisplayData_Sorting(t *testing.T) {
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			nodeDisplayData, _ := assessNodesStatus(tc.args.cv, tc.args.pool, tc.args.nodes, tc.args.machineConfigs)
+			nodeDisplayData, _ := assessNodesStatus(tc.args.cv, tc.args.pool, tc.args.nodes, tc.args.machineConfigs, false)
 			if diff := cmp.Diff(tc.expectedNodeDisplayData, nodeDisplayData, allowUnexportedWorkerPools); diff != "" {
 				t.Errorf("nodeDisplayData differ from expected:\n%s", diff)
 			}
@@ -970,12 +967,13 @@ func Test_nodeInsights(t *testing.T) {
 	mcpMaster := mcp("master").setMachineConfig("new").machineConfigPool
 	mcpWorker := mcp("worker").setMachineConfig("new").machineConfigPool
 	type args struct {
-		pool          mcfgv1.MachineConfigPool
-		node          corev1.Node
-		reason        string
-		isUnavailable bool
-		isUpdating    bool
-		isDegraded    bool
+		pool             mcfgv1.MachineConfigPool
+		node             corev1.Node
+		reason           string
+		isUnavailable    bool
+		isUpdating       bool
+		isDegraded       bool
+		unavailableSince time.Time
 	}
 	testCases := []struct {
 		name                  string
@@ -993,17 +991,19 @@ func Test_nodeInsights(t *testing.T) {
 		{
 			name: "node is updated - unavailable - master pool",
 			args: args{
-				pool:          mcpMaster,
-				node:          node("a").updated(mcNew.Name, mcNew.Name).unavailable().node,
-				reason:        "Node is unavailable",
-				isUnavailable: true,
+				pool:             mcpMaster,
+				node:             node("a").updated(mcNew.Name, mcNew.Name).unavailable().node,
+				reason:           "Node is unavailable",
+				isUnavailable:    true,
+				unavailableSince: time.Date(2009, 11, 17, 20, 34, 58, 651387237, time.UTC),
 			},
 			expectedUpdateInsight: []updateInsight{
 				{
+					startedAt: time.Date(2009, 11, 17, 20, 34, 58, 651387237, time.UTC),
 					impact: updateInsightImpact{
 						level:       warningImpactLevel,
 						impactType:  updateSpeedImpactType,
-						summary:     "Node a is unavailable",
+						summary:     "Node is unavailable",
 						description: "Node is unavailable",
 					},
 					scope: updateInsightScope{
@@ -1029,7 +1029,7 @@ func Test_nodeInsights(t *testing.T) {
 					impact: updateInsightImpact{
 						level:       warningImpactLevel,
 						impactType:  updateSpeedImpactType,
-						summary:     "Node a is unavailable",
+						summary:     "Node is unavailable",
 						description: "Node is unavailable",
 					},
 					scope: updateInsightScope{
@@ -1073,9 +1073,9 @@ func Test_nodeInsights(t *testing.T) {
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			updateInsights := nodeInsights(tc.args.pool, tc.args.node, tc.args.reason, tc.args.isUnavailable, tc.args.isUpdating, tc.args.isDegraded)
+			updateInsights := nodeInsights(tc.args.pool, tc.args.node.Name, tc.args.reason, tc.args.reason, tc.args.isUnavailable, tc.args.isUpdating, tc.args.isDegraded, tc.args.unavailableSince)
 			if diff := cmp.Diff(tc.expectedUpdateInsight, updateInsights, allowUnexportedInsightStructs); diff != "" {
-				t.Errorf("updateInsight differ from expected:\n%s", diff)
+				t.Errorf("%s: updateInsight differ from expected:\n%s", tc.name, diff)
 			}
 		})
 	}
