@@ -26,6 +26,7 @@ import (
 	imagereference "github.com/openshift/library-go/pkg/image/reference"
 
 	"github.com/openshift/oc/pkg/cli/admin/upgrade/channel"
+	"github.com/openshift/oc/pkg/cli/admin/upgrade/recommend"
 	"github.com/openshift/oc/pkg/cli/admin/upgrade/rollback"
 	"github.com/openshift/oc/pkg/cli/admin/upgrade/status"
 )
@@ -92,9 +93,15 @@ func New(f kcmdutil.Factory, streams genericiooptions.IOStreams) *cobra.Command
 			regardless of these concerns, use --allow-upgrade-with-warnings.
 
 			Cluster-side guards include checks for release verification and upgradeable conditions.
-			Again, it is usually best to give these conditions time to resolve, or to actively work to
-			resolve them.  But if you decide to trigger the update regardless of these concerns,
-			use --force, which is passed through to ClusterVersion's spec.desiredUpdate.force.
+			You can push through them with --force, which is passed through to ClusterVersion's
+			spec.desiredUpdate.force, but only do that if:
+
+			* you are testing unsigned release images in short-lived test clusters or
+			* you are working around a known bug in the cluster-version operator and you have verified
+			  the authenticity of the provided image yourself.
+
+			The provided image will run with full administrative access to the cluster. Do not use
+			--force with images that come from unknown or potentially malicious sources.
 		`),
 		Run: func(cmd *cobra.Command, args []string) {
 			kcmdutil.CheckErr(o.Complete(f, cmd, args))
@@ -104,24 +111,22 @@ func New(f kcmdutil.Factory, streams genericiooptions.IOStreams) *cobra.Command
 	flags := cmd.Flags()
 	flags.StringVar(&o.To, "to", o.To, "Specify the version to upgrade to. The version must be on the list of available updates.")
 	flags.StringVar(&o.ToImage, "to-image", o.ToImage, "Provide a release image to upgrade to.")
-	flags.BoolVar(&o.ToLatestAvailable, "to-latest", o.ToLatestAvailable, "Use the next available version.")
+	flags.BoolVar(&o.ToLatestAvailable, "to-latest", o.ToLatestAvailable, "Use the latest (highest Semantic Version) available version.")
 	flags.BoolVar(&o.ToMultiArch, "to-multi-arch", o.ToMultiArch, "Upgrade current version to multi architecture.")
 	flags.BoolVar(&o.Clear, "clear", o.Clear, "If an upgrade has been requested but not yet downloaded, cancel the update. This has no effect once the update has started.")
-	flags.BoolVar(&o.Force, "force", o.Force, "Upgrade regardless of cluster-side guard failures, such as release verification or upgradeable conditions.")
+	flags.BoolVar(&o.Force, "force", o.Force, "Upgrade regardless of cluster-side guard failures, such as release verification or upgradeable conditions. Only use this if you are testing unsigned release images or you are working around a known bug in the cluster-version operator and you have verified the authenticity of the provided image yourself.")
 	flags.BoolVar(&o.AllowExplicitUpgrade, "allow-explicit-upgrade", o.AllowExplicitUpgrade, "Upgrade even if the upgrade target is not listed in the available versions list.")
 	flags.BoolVar(&o.AllowUpgradeWithWarnings, "allow-upgrade-with-warnings", o.AllowUpgradeWithWarnings, "Upgrade regardless of client-side guard failures, such as upgrades in progress or failing clusters.")
 	flags.BoolVar(&o.IncludeNotRecommended, "include-not-recommended", o.IncludeNotRecommended, "Display additional updates which are not recommended based on your cluster configuration.")
 	flags.BoolVar(&o.AllowNotRecommended, "allow-not-recommended", o.AllowNotRecommended, "Allows upgrade to a version when it is supported but not recommended for updates.")
 
-	if kcmdutil.FeatureGate("OC_ENABLE_CMD_UPGRADE_STATUS").IsEnabled() {
-		cmd.AddCommand(status.New(f, streams))
-	}
-
 	cmd.AddCommand(channel.New(f, streams))
+	cmd.AddCommand(status.New(f, streams))
 
 	if kcmdutil.FeatureGate("OC_ENABLE_CMD_UPGRADE_ROLLBACK").IsEnabled() {
 		cmd.AddCommand(rollback.New(f, streams))
 	}
+	cmd.AddCommand(recommend.New(f, streams))
 
 	return cmd
 }
@@ -150,8 +155,8 @@ func (o *Options) Complete(f kcmdutil.Factory, cmd *cobra.Command, args []string
 	if o.Clear && (len(o.ToImage) > 0 || len(o.To) > 0 || o.ToLatestAvailable || o.ToMultiArch) {
 		return fmt.Errorf("--clear may not be specified with any other flags")
 	}
-	if o.ToMultiArch && (len(o.To) > 0 || len(o.ToImage) > 0) {
-		return fmt.Errorf("--to-multi-arch may not be used with --to or --to-image")
+	if o.ToMultiArch && (len(o.To) > 0 || len(o.ToImage) > 0 || o.ToLatestAvailable) {
+		return fmt.Errorf("--to-multi-arch may not be used with --to, --to-image, or --to-latest")
 	}
 	if len(o.To) > 0 && len(o.ToImage) > 0 {
 		return fmt.Errorf("only one of --to or --to-image may be provided")
@@ -237,8 +242,17 @@ func (o *Options) Run() error {
 			fmt.Fprintf(o.ErrOut, "warning: --allow-upgrade-with-warnings is bypassing: %s\n", err)
 		}
 
-		if err := patchDesiredUpdate(ctx, &configv1.Update{Architecture: configv1.ClusterVersionArchitectureMulti,
-			Version: cv.Status.Desired.Version}, o.Client, cv.Name); err != nil {
+		update := &configv1.Update{
+			Architecture: configv1.ClusterVersionArchitectureMulti,
+			Version:      cv.Status.Desired.Version,
+		}
+
+		if o.Force {
+			update.Force = true
+			fmt.Fprintln(o.ErrOut, "warning: --force overrides cluster verification of your supplied release image and waives any update precondition failures. Only use this if you are testing unsigned release images or you are working around a known bug in the cluster-version operator and you have verified the authenticity of the provided image yourself.")
+		}
+
+		if err := patchDesiredUpdate(ctx, update, o.Client, cv.Name); err != nil {
 
 			return err
 		}
@@ -367,7 +381,7 @@ func (o *Options) Run() error {
 
 		if o.Force {
 			update.Force = true
-			fmt.Fprintln(o.ErrOut, "warning: --force overrides cluster verification of your supplied release image and waives any update precondition failures.")
+			fmt.Fprintln(o.ErrOut, "warning: --force overrides cluster verification of your supplied release image and waives any update precondition failures. Only use this if you are testing unsigned release images or you are working around a known bug in the cluster-version operator and you have verified the authenticity of the provided image yourself.")
 		}
 
 		if err := checkForUpgrade(cv); err != nil {
