@@ -2,6 +2,7 @@ package debug
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"os"
 	"reflect"
@@ -41,7 +42,6 @@ import (
 	"k8s.io/kubectl/pkg/scheme"
 	"k8s.io/kubectl/pkg/util/interrupt"
 	"k8s.io/kubectl/pkg/util/templates"
-	"k8s.io/kubectl/pkg/util/term"
 	"k8s.io/pod-security-admission/api"
 
 	appsv1 "github.com/openshift/api/apps/v1"
@@ -61,6 +61,7 @@ import (
 )
 
 const (
+	debugPodLabelManagedBy            = "debug.openshift.io/managed-by"
 	debugPodAnnotationSourceContainer = "debug.openshift.io/source-container"
 	debugPodAnnotationSourceResource  = "debug.openshift.io/source-resource"
 	// containerResourcesAnnotationPrefix contains resource annotation prefix that will be used by CRI-O to set cpu shares
@@ -302,7 +303,7 @@ func (o *DebugOptions) Complete(cmd *cobra.Command, f kcmdutil.Factory, args []s
 		o.Attach.TTY = false
 		o.Attach.Stdin = false
 	default:
-		o.Attach.TTY = term.IsTerminal(o.In)
+		o.Attach.TTY = printers.IsTerminal(o.In)
 		klog.V(4).Infof("Defaulting TTY to %t", o.Attach.TTY)
 	}
 	if o.NoStdin {
@@ -549,7 +550,7 @@ func (o *DebugOptions) RunDebug() error {
 			}
 			if o.IsNode {
 				if !(template.Spec.OS != nil && template.Spec.OS.Name == corev1.Windows) {
-					fmt.Fprintf(o.ErrOut, "To use host binaries, run `chroot /host`\n")
+					fmt.Fprintf(o.ErrOut, "To use host binaries, run `chroot /host`. Instead, if you need to access host namespaces, run `nsenter -a -t 1`.\n")
 				}
 			}
 		}
@@ -611,7 +612,7 @@ func (o *DebugOptions) RunDebug() error {
 			if len(o.NodeName) > 0 {
 				msg += fmt.Sprintf(" on node %q", o.NodeName)
 			}
-			return fmt.Errorf(msg)
+			return errors.New(msg)
 			// switch to logging output
 		case err == krun.ErrPodCompleted, err == conditions.ErrContainerTerminated:
 			resultPod, ok := containerRunningEvent.Object.(*corev1.Pod)
@@ -625,7 +626,7 @@ func (o *DebugOptions) RunDebug() error {
 						if len(o.NodeName) > 0 {
 							msg += fmt.Sprintf(" on node %q", o.NodeName)
 						}
-						return fmt.Errorf(msg)
+						return errors.New(msg)
 					}
 				}
 			}
@@ -900,13 +901,8 @@ func (o *DebugOptions) transformPodForDebug(annotations map[string]string) (*cor
 	for k, v := range annotations {
 		pod.Annotations[k] = v
 	}
-	if o.KeepLabels {
-		if pod.Labels == nil {
-			pod.Labels = make(map[string]string)
-		}
-	} else {
-		pod.Labels = map[string]string{}
-	}
+
+	pod.Labels = createLabelMap(pod.Labels, o.KeepLabels)
 
 	pod.ResourceVersion = ""
 	pod.Spec.RestartPolicy = corev1.RestartPolicyNever
@@ -921,6 +917,19 @@ func (o *DebugOptions) transformPodForDebug(annotations map[string]string) (*cor
 	return pod, originalCommand
 }
 
+func createLabelMap(currentLabels map[string]string, keepLabels bool) map[string]string {
+	if keepLabels && currentLabels != nil {
+		return currentLabels
+	}
+
+	newLabels := make(map[string]string)
+	if val, ok := currentLabels[debugPodLabelManagedBy]; ok {
+		newLabels[debugPodLabelManagedBy] = val
+	}
+
+	return newLabels
+}
+
 // createPod creates the debug pod, and will attempt to delete an existing debug
 // pod with the same name, but will return an error in any other case.
 func (o *DebugOptions) createPod(pod *corev1.Pod) (*corev1.Pod, error) {
@@ -1053,6 +1062,11 @@ func (o *DebugOptions) approximatePodTemplateForObject(object runtime.Object) (*
 		isTrue := true
 		hostPathType := corev1.HostPathDirectory
 		template := &corev1.PodTemplateSpec{
+			ObjectMeta: metav1.ObjectMeta{
+				Labels: map[string]string{
+					debugPodLabelManagedBy: "oc-debug",
+				},
+			},
 			Spec: corev1.PodSpec{
 				NodeName:    t.Name,
 				HostNetwork: true,
@@ -1091,6 +1105,11 @@ func (o *DebugOptions) approximatePodTemplateForObject(object runtime.Object) (*
 								Name:  "TMOUT",
 								Value: "900",
 							},
+							{
+								//  to collect more sos report requires this env var is set
+								Name:  "HOST",
+								Value: "/host",
+							},
 						},
 					},
 				},
