@@ -17,21 +17,21 @@ import (
 
 	"github.com/pkg/browser"
 
+	userv1 "github.com/openshift/api/user/v1"
+	projectv1typedclient "github.com/openshift/client-go/project/clientset/versioned/typed/project/v1"
+	"github.com/openshift/library-go/pkg/oauth/tokenrequest"
+	"github.com/openshift/library-go/pkg/oauth/tokenrequest/challengehandlers"
 	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/cli-runtime/pkg/genericiooptions"
+	"k8s.io/cli-runtime/pkg/printers"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/pkg/apis/clientauthentication"
 	restclient "k8s.io/client-go/rest"
 	kclientcmd "k8s.io/client-go/tools/clientcmd"
 	kclientcmdapi "k8s.io/client-go/tools/clientcmd/api"
 	"k8s.io/klog/v2"
-	kterm "k8s.io/kubectl/pkg/util/term"
-
-	projectv1typedclient "github.com/openshift/client-go/project/clientset/versioned/typed/project/v1"
-	"github.com/openshift/library-go/pkg/oauth/tokenrequest"
-	"github.com/openshift/library-go/pkg/oauth/tokenrequest/challengehandlers"
 
 	occhallengers "github.com/openshift/oc/pkg/helpers/authchallengers"
 	ocerrors "github.com/openshift/oc/pkg/helpers/errors"
@@ -80,12 +80,13 @@ type LoginOptions struct {
 	CertFile string
 	KeyFile  string
 
-	OIDCExecPluginType string
-	OIDCClientID       string
-	OIDCClientSecret   string
-	OIDCExtraScopes    []string
-	OIDCIssuerURL      string
-	OIDCCAFile         string
+	OIDCExecPluginType  string
+	OIDCClientID        string
+	OIDCClientSecret    string
+	OIDCExtraScopes     []string
+	OIDCIssuerURL       string
+	OIDCCAFile          string
+	OIDCAutoOpenBrowser bool
 
 	Token string
 
@@ -95,6 +96,10 @@ type LoginOptions struct {
 	RequestTimeout time.Duration
 
 	genericiooptions.IOStreams
+
+	// whoAmIFunc is used to mock project.WhoAmI. It's only being used in tests.
+	// See LoginOptions.whoAmI for details.
+	whoAmIFunc func(clientConfig *restclient.Config) (*userv1.User, error)
 }
 
 type passwordPrompter func(r io.Reader, w io.Writer, format string, a ...interface{}) string
@@ -133,11 +138,10 @@ func (o *LoginOptions) getClientConfig() (*restclient.Config, error) {
 
 	if len(o.Server) == 0 {
 		// we need to have a server to talk to
-		if kterm.IsTerminal(o.In) {
+		if printers.IsTerminal(o.In) {
 			for !o.serverProvided() {
-				defaultServer := defaultClusterURL
-				promptMsg := fmt.Sprintf("Server [%s]: ", defaultServer)
-				o.Server = term.PromptForStringWithDefault(o.In, o.Out, defaultServer, promptMsg)
+				o.Server = term.PromptForStringWithDefault(
+					o.In, o.Out, defaultClusterURL, "Server [%s]: ", defaultClusterURL)
 			}
 		}
 	}
@@ -236,7 +240,7 @@ func (o *LoginOptions) gatherAuthInfo() error {
 	// if a token were explicitly provided, try to use it
 	if o.tokenProvided() {
 		clientConfig.BearerToken = o.Token
-		me, err := project.WhoAmI(clientConfig)
+		me, err := o.whoAmI(clientConfig)
 		if err != nil {
 			if kerrors.IsUnauthorized(err) {
 				return fmt.Errorf("The token provided is invalid or expired.\n\n")
@@ -261,13 +265,18 @@ func (o *LoginOptions) gatherAuthInfo() error {
 			if matchingClusters.Has(context.Cluster) {
 				clientcmdConfig := kclientcmd.NewDefaultClientConfig(kubeconfig, &kclientcmd.ConfigOverrides{CurrentContext: key})
 				if kubeconfigClientConfig, err := clientcmdConfig.ClientConfig(); err == nil {
-					if me, err := project.WhoAmI(kubeconfigClientConfig); err == nil && (o.Username == me.Name) {
+					if me, err := o.whoAmI(kubeconfigClientConfig); err == nil && (o.Username == me.Name) {
 						clientConfig.BearerToken = kubeconfigClientConfig.BearerToken
 						clientConfig.CertFile = kubeconfigClientConfig.CertFile
 						clientConfig.CertData = kubeconfigClientConfig.CertData
 						clientConfig.KeyFile = kubeconfigClientConfig.KeyFile
 						clientConfig.KeyData = kubeconfigClientConfig.KeyData
 
+						// Preserve ExecProvider configuration.
+						if kubeconfigClientConfig.ExecProvider != nil {
+							clientConfig.ExecProvider = kubeconfigClientConfig.ExecProvider.DeepCopy()
+						}
+
 						o.Config = clientConfig
 
 						fmt.Fprintf(o.Out, "Logged into %q as %q using existing credentials.\n\n", o.Config.Host, o.Username)
@@ -286,7 +295,7 @@ func (o *LoginOptions) gatherAuthInfo() error {
 		}
 
 		clientConfig.ExecProvider = execProvider
-		me, err := project.WhoAmI(clientConfig)
+		me, err := o.whoAmI(clientConfig)
 		if err != nil {
 			return err
 		}
@@ -308,8 +317,14 @@ func (o *LoginOptions) gatherAuthInfo() error {
 	if o.WebLogin {
 		loginURLHandler := func(u *url.URL) error {
 			loginURL := u.String()
-			fmt.Fprintf(o.Out, "Opening login URL in the default browser: %s\n", loginURL)
-			return browser.OpenURL(loginURL)
+			if !o.OIDCAutoOpenBrowser {
+				fmt.Fprintf(o.Out, "Please visit the following URL in your browser: %s\n", loginURL)
+				fmt.Fprintf(o.Out, "The callback server is listening and will receive the authentication response.\n")
+				return nil
+			} else {
+				fmt.Fprintf(o.Out, "Opening login URL in the default browser: %s\n", loginURL)
+				return browser.OpenURL(loginURL)
+			}
 		}
 		token, err = tokenrequest.RequestTokenWithLocalCallback(o.Config, loginURLHandler, int(o.CallbackPort))
 	} else {
@@ -321,7 +336,7 @@ func (o *LoginOptions) gatherAuthInfo() error {
 
 	clientConfig.BearerToken = token
 
-	me, err := project.WhoAmI(clientConfig)
+	me, err := o.whoAmI(clientConfig)
 	if err != nil {
 		return err
 	}
@@ -364,6 +379,10 @@ func (o *LoginOptions) prepareBuiltinExecPlugin() (*kclientcmdapi.ExecConfig, er
 		execProvider.Args = append(execProvider.Args, fmt.Sprintf("--certificate-authority=%s", o.OIDCCAFile))
 	}
 
+	if o.OIDCAutoOpenBrowser {
+		execProvider.Args = append(execProvider.Args, "--auto-open-browser")
+	}
+
 	return execProvider, nil
 }
 
@@ -453,7 +472,7 @@ func (o *LoginOptions) gatherProjectInfo() error {
 			return err
 		}
 		msg := ocerrors.NoProjectsExistMessage(canRequest)
-		fmt.Fprintf(o.Out, msg)
+		fmt.Fprint(o.Out, msg)
 		o.Project = ""
 
 	case 1:
@@ -555,7 +574,7 @@ func (o *LoginOptions) SaveConfig() (bool, error) {
 		}
 
 		out := &bytes.Buffer{}
-		fmt.Fprintf(out, ocerrors.ErrKubeConfigNotWriteable(o.PathOptions.GetDefaultFilename(), o.PathOptions.IsExplicitFile(), err).Error())
+		fmt.Fprint(out, ocerrors.ErrKubeConfigNotWriteable(o.PathOptions.GetDefaultFilename(), o.PathOptions.IsExplicitFile(), err).Error())
 		return false, fmt.Errorf("%v", out)
 	}
 
@@ -567,6 +586,13 @@ func (o *LoginOptions) SaveConfig() (bool, error) {
 	return created, nil
 }
 
+func (o *LoginOptions) whoAmI(clientConfig *restclient.Config) (*userv1.User, error) {
+	if o.whoAmIFunc != nil {
+		return o.whoAmIFunc(clientConfig)
+	}
+	return project.WhoAmI(clientConfig)
+}
+
 func (o *LoginOptions) usernameProvided() bool {
 	return len(o.Username) > 0
 }
