@@ -3,6 +3,7 @@ package newapp
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"os"
@@ -23,7 +24,7 @@ import (
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/apimachinery/pkg/util/errors"
+	kutilerrors "k8s.io/apimachinery/pkg/util/errors"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
@@ -486,7 +487,7 @@ func (o *AppOptions) RunNewApp() error {
 			if len(t.Spec.Host) > 0 {
 				var route *routev1.Route
 				//check if route processing was completed and host field is prepopulated by router
-				err := wait.PollImmediate(500*time.Millisecond, RoutePollTimeout, func() (bool, error) {
+				err := wait.PollUntilContextTimeout(context.TODO(), 500*time.Millisecond, RoutePollTimeout, true, func(ctx context.Context) (bool, error) {
 					route, err = config.RouteClient.Routes(t.Namespace).Get(context.TODO(), t.Name, metav1.GetOptions{})
 					if err != nil {
 						return false, fmt.Errorf("Error while polling route %s", t.Name)
@@ -557,7 +558,7 @@ func followInstallation(config *newcmd.AppConfig, clientGetter genericclioptions
 	// we cannot retrieve logs until the pod is out of pending
 	// TODO: move this to the server side
 	podClient := config.KubeClient.CoreV1().Pods(pod.Namespace)
-	if err := wait.PollImmediate(500*time.Millisecond, 60*time.Second, installationStarted(podClient, pod.Name, config.KubeClient.CoreV1().Secrets(pod.Namespace))); err != nil {
+	if err := wait.PollUntilContextTimeout(context.TODO(), 500*time.Millisecond, 60*time.Second, true, installationStarted(podClient, pod.Name, config.KubeClient.CoreV1().Secrets(pod.Namespace))); err != nil {
 		return err
 	}
 
@@ -576,7 +577,7 @@ func followInstallation(config *newcmd.AppConfig, clientGetter genericclioptions
 	logErr := opts.RunLogs()
 
 	// status of the pod may take tens of seconds to propagate
-	if err := wait.PollImmediate(500*time.Millisecond, 30*time.Second, installationComplete(podClient, pod.Name, config.Out)); err != nil {
+	if err := wait.PollUntilContextTimeout(context.TODO(), 500*time.Millisecond, 30*time.Second, true, installationComplete(podClient, pod.Name, config.Out)); err != nil {
 		if err == wait.ErrWaitTimeout {
 			if logErr != nil {
 				// output the log error if one occurred
@@ -591,8 +592,8 @@ func followInstallation(config *newcmd.AppConfig, clientGetter genericclioptions
 	return nil
 }
 
-func installationStarted(c corev1typedclient.PodInterface, name string, s corev1typedclient.SecretInterface) wait.ConditionFunc {
-	return func() (bool, error) {
+func installationStarted(c corev1typedclient.PodInterface, name string, s corev1typedclient.SecretInterface) wait.ConditionWithContextFunc {
+	return func(ctx context.Context) (bool, error) {
 		pod, err := c.Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil {
 			return false, err
@@ -613,8 +614,8 @@ func installationStarted(c corev1typedclient.PodInterface, name string, s corev1
 	}
 }
 
-func installationComplete(c corev1typedclient.PodInterface, name string, out io.Writer) wait.ConditionFunc {
-	return func() (bool, error) {
+func installationComplete(c corev1typedclient.PodInterface, name string, out io.Writer) wait.ConditionWithContextFunc {
+	return func(ctx context.Context) (bool, error) {
 		pod, err := c.Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil {
 			if kapierrors.IsNotFound(err) {
@@ -724,23 +725,23 @@ func CompleteAppConfig(config *newcmd.AppConfig, f kcmdutil.Factory, c *cobra.Co
 			fmt.Fprintf(buf, "%s:\n", argName)
 			for _, classErr := range config.EnvironmentClassificationErrors {
 				if classErr.Value != nil {
-					fmt.Fprintf(buf, fmt.Sprintf("%s:  %v\n", classErr.Key, classErr.Value))
+					fmt.Fprintf(buf, "%s:  %v\n", classErr.Key, classErr.Value)
 				} else {
-					fmt.Fprintf(buf, fmt.Sprintf("%s\n", classErr.Key))
+					fmt.Fprintf(buf, "%s\n", classErr.Key)
 				}
 			}
 			for _, classErr := range config.SourceClassificationErrors {
-				fmt.Fprintf(buf, fmt.Sprintf("%s:  %v\n", classErr.Key, classErr.Value))
+				fmt.Fprintf(buf, "%s:  %v\n", classErr.Key, classErr.Value)
 			}
 			for _, classErr := range config.TemplateClassificationErrors {
-				fmt.Fprintf(buf, fmt.Sprintf("%s:  %v\n", classErr.Key, classErr.Value))
+				fmt.Fprintf(buf, "%s:  %v\n", classErr.Key, classErr.Value)
 			}
 			for _, classErr := range config.ComponentClassificationErrors {
-				fmt.Fprintf(buf, fmt.Sprintf("%s:  %v\n", classErr.Key, classErr.Value))
+				fmt.Fprintf(buf, "%s:  %v\n", classErr.Key, classErr.Value)
 			}
 			fmt.Fprintln(buf)
 		}
-		return kcmdutil.UsageErrorf(c, heredoc.Docf(buf.String()))
+		return kcmdutil.UsageErrorf(c, "%s", heredoc.Doc(buf.String()))
 	}
 
 	if config.AllowMissingImages && config.AsSearch {
@@ -891,7 +892,7 @@ func HandleError(err error, commandPath string, config *newcmd.AppConfig, transf
 		return nil
 	}
 	errs := []error{err}
-	if agg, ok := err.(errors.Aggregate); ok {
+	if agg, ok := err.(kutilerrors.Aggregate); ok {
 		errs = agg.Errors()
 	}
 	groups := ErrorGroups{}
@@ -904,7 +905,7 @@ func HandleError(err error, commandPath string, config *newcmd.AppConfig, transf
 		if len(group.classification) > 0 {
 			fmt.Fprintln(buf)
 		}
-		fmt.Fprintf(buf, group.classification)
+		fmt.Fprint(buf, group.classification)
 		if len(group.suggestion) > 0 {
 			if len(group.classification) > 0 {
 				fmt.Fprintln(buf)
@@ -913,7 +914,7 @@ func HandleError(err error, commandPath string, config *newcmd.AppConfig, transf
 		}
 		fmt.Fprint(buf, group.suggestion)
 	}
-	return fmt.Errorf(buf.String())
+	return errors.New(buf.String())
 }
 
 type ErrorGroup struct {
@@ -1068,13 +1069,14 @@ func TransformRunError(err error, commandPath string, groups ErrorGroups, config
 	switch err {
 	case errNoTokenAvailable:
 		// TODO: improve by allowing token generation
-		groups.Add("", "", "", fmt.Errorf("to install components you must be logged in with an OAuth token (instead of only a certificate)"))
+		groups.Add("", "", "", errors.New("to install components you must be logged in with an OAuth token (instead of only a certificate)"))
 	case newcmd.ErrNoInputs:
 		// TODO: suggest things to the user
-		groups.Add("", "", "", UsageError(commandPath, newAppNoInput))
+		groups.Add("", "", "", UsageError(commandPath, "%s", newAppNoInput))
 	default:
 		if runtime.IsNotRegisteredError(err) {
-			groups.Add("", "", "", fmt.Errorf(fmt.Sprintf("The template contained an object type unknown to `oc new-app`.  Use `oc process -f <template> | oc create -f -` instead.  Error details: %v", err)))
+			groups.Add("", "", "", fmt.Errorf(
+				"The template contained an object type unknown to `oc new-app`.  Use `oc process -f <template> | oc create -f -` instead.  Error details: %v", err))
 		} else {
 			groups.Add("", "", "", err)
 		}
