@@ -2,6 +2,7 @@ package process
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"math/rand"
@@ -12,7 +13,7 @@ import (
 	"github.com/spf13/cobra"
 
 	corev1 "k8s.io/api/core/v1"
-	"k8s.io/apimachinery/pkg/api/errors"
+	kapierrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
@@ -186,7 +187,7 @@ func (p *processPrinter) PrintObj(obj runtime.Object, out io.Writer) error {
 			return fmt.Errorf("error describing %q: %v\n", templateObj.Name, err)
 		}
 
-		fmt.Fprintf(out, s)
+		fmt.Fprint(out, s)
 		return nil
 	}
 
@@ -258,7 +259,7 @@ func (o *ProcessOptions) Complete(f kcmdutil.Factory, cmd *cobra.Command, args [
 		o.templateProcessor = func(template *templatev1.Template) (*templatev1.Template, error) {
 			t, err := templateProcessorClient.Process(template)
 			if err != nil {
-				if err, ok := err.(*errors.StatusError); ok && err.ErrStatus.Details != nil {
+				if err, ok := err.(*kapierrors.StatusError); ok && err.ErrStatus.Details != nil {
 					errstr := "unable to process template\n"
 					for _, cause := range err.ErrStatus.Details.Causes {
 						errstr += fmt.Sprintf("  %s\n", cause.Message)
@@ -270,7 +271,7 @@ func (o *ProcessOptions) Complete(f kcmdutil.Factory, cmd *cobra.Command, args [
 						errstr += fmt.Sprintf("  %v\n", err)
 					}
 
-					return nil, fmt.Errorf(errstr)
+					return nil, errors.New(errstr)
 				}
 
 				return nil, fmt.Errorf("unable to process template: %v\n", err)
@@ -345,7 +346,7 @@ func (o *ProcessOptions) RunProcess() error {
 
 		templateObj, err := o.templateClient.Templates(sourceNamespace).Get(context.TODO(), storedTemplate, metav1.GetOptions{})
 		if err != nil {
-			if errors.IsNotFound(err) {
+			if kapierrors.IsNotFound(err) {
 				return fmt.Errorf("template %q could not be found", storedTemplate)
 			}
 			return err
@@ -476,7 +477,7 @@ func processTemplateLocally(tpl *templatev1.Template) (*templatev1.Template, err
 		"expression": generator.NewExpressionValueGenerator(rand.New(rand.NewSource(time.Now().UnixNano()))),
 	})
 	if errs := processor.Process(tpl); len(errs) > 0 {
-		return nil, errors.NewInvalid(octemplateapi.Kind("Template"), tpl.Name, errs)
+		return nil, kapierrors.NewInvalid(octemplateapi.Kind("Template"), tpl.Name, errs)
 	}
 
 	return tpl, nil
