@@ -341,7 +341,7 @@ func (c *AppConfig) tryToAddSourceArguments(s string) bool {
 	}
 	c.SourceClassificationErrors[s] = ArgumentClassificationError{
 		Key:   "is not a Git repository",
-		Value: fmt.Errorf(errStr),
+		Value: errors.New(errStr),
 	}
 
 	return false
